// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
//
// You can also hide map Icons by setting the team, eg.  %obj.setTeam(10) to a value
// outside of 0-3
//
//  Setting it within the range of 0-3 will cause it to display in the team color
//
// %obj.setOwnTeamIcon(true) will hide that icon from players not on your team
//
// map icons are created in client/scripts/mapIcons.cs
//
// The commander map itself can be navigated by left-mouse-drag
// Zoom is accomplished by middle-mouse-wheel or middle-mouse-drag (vertical motion)
//
// Mouse right click is captured as well and calls function GuiCommanderHudRightClick
// which gets a variable corresponding to the worldpos of where the mouse is
//
// Clicking an icon will call function GuiCommanderHudIconClick and will pass the client
// obj.getId() of the object thats was clicked
//
// You can call CommanderMapGui.clickLastSelection() to re-click whatever you last clicked on
//
// onlyShowThisTeam(team) will hide icons belonging to the other team.  onlyShowThisTeam(0) will
// show all icons so use it to reset a previous call
//
// getMyTeam() will return your current team but will obviously only work once you have spawned
// and therefore been assigned to a team on the server.
//
// setting SpawnInSweepCam to true or false on the control itself will enable/disable the spawn in sweeping camera effect
// default is off

function toggleCommanderMapGui()
{
 CommanderMapGui.pickSpawn = 0;
  if (Canvas.getContent() == PlayGui.getId())
    {
     Canvas.setContent(CommanderMapGui);
     AutoSpawnButton.setVisible(false);
    }
  else
    Canvas.setContent(PlayGui);
}

function CommanderMapView::onWake(%this)
{
   // just update the action map here
   moveMap.push();
   SetTeamIcon();
   CommanderMapView.updateShowItems();
}

function CommanderMapView::onSleep(%this)
  {
   // restore old maps.
   moveMap.pop();
  }

// set the commandermapview to show all teams on the map
function CommanderMapView::setShowAllTeams(%this)
  {
   %this.ShowMyTeamOnly = "false";
  }

function CommanderMapView::setShowOnlyOwnTeam(%this)
  {
   %this.ShowMyTeamOnly = "true";
  }

// update commander map view settings to display things correctly
// this is called when a player switches teams, and as a part of onWake()
function CommanderMapView::updateShowItems(%this)
  {
   if ( %this.showMyTeamOnly $= "true")
     {
      // try various ways to get the teamID
      %team = CommanderMapView.getMyTeam();
      if (%team==0)
        {
         %team = Canvas.connection.teamID;
         if (%team ==0)
           MessageBoxOK("Error", "Unable to get team ID");
        }

      CommanderMapView.onlyShowThisTeam(%team);
     }
   else
     {
      CommanderMapView.onlyShowThisTeam(0);
     }
  }

// Below function called from C++ in guicomandermap.cc
function GuiCommanderHudClose()
{
	PlayGuiEverything.visible = 0;
	$ignoreOnAwake = 1;
	Canvas.setContent(PlayGui);
}

// setup commander GUI to pick a spawn point.
function pickSpawnPoint()
  {
   CommanderMapGui.pickSpawn = 1;
   Canvas.setContent(CommanderMapGui);
   AutoSpawnButton.setVisible(true);
   CommanderMapGui.useExistingSpawnPoint = 0;
   CommanderMapView.updateShowItems();
  }

// spawn point was clicked on.  if we are in 'pick spawn point' mode then put
// the commander map gui away and create a player
function clientCmdSpawnPointChosen()
  {
   if (CommanderMapGui.pickSpawn == 1)
     {
      if (CommanderMapGui.useExistingSpawnPoint == 1)
        {
         error("Ignoring duplicate spawn");
         return;
        }
      CommanderMapGui.useExistingSpawnPoint = 1;

      commandtoserver('JoinPlayerClass', $Client::ClassToJoin );
      CommanderMapView.clickLastSelection();
     }
  }

//--- OBJECT WRITE END ---
$PanFactor = 50;
$CommandMapZoomFactor = 1;
$CommandMapY = 0;
$CommandMapX = 0;

function panMapTo(){
   CommanderMapView.panTo($CommandMapX,$CommandMapY);
}

function panMapPlayer() {

  CommanderMapView.centerOnMe();
}

function panMapCenter() {
   $CommandMapX = 0;
   $CommandMapY = 0;
   panMapTo();
}

function panMapUp() {
   $CommandMapY = $CommandMapY + $PanFactor;
   panMapTo();
}

function panMapDown() {
   $CommandMapY = $CommandMapY - $PanFactor;
   panMapTo();
}

function panMapLeft() {
   $CommandMapX = $CommandMapX - $PanFactor;
   panMapTo();
}

function panMapRight() {
   $CommandMapX = $CommandMapX + $PanFactor;
   panMapTo();
}

function ZoomIn(){
  $CommandMapZoomFactor= $CommandMapZoomFactor- 0.2;
  if($CommandMapZoomFactor<0.2)$CommandMapZoomFactor = 0.2;
  panMapTo();
}


function ZoomOut(){
  $CommandMapZoomFactor= $CommandMapZoomFactor+ 0.2;
  if($CommandMapZoomFactor>2.2)$CommandMapZoomFactor = 2.2;
  panMapTo();
}

function cmap(){
  exec("starter.combat/client/scripts/commandermapgui.cs");
}

function panMapRed()
{
   // set the pnt
   $CommandMapX = $RedBaseXpos_;
   $CommandMapY = $RedBaseYpos_;
   panMapTo();
}
function panMapBlue()
{
   // set the pnt
   $CommandMapX = $BlueBaseXpos_;
   $CommandMapY = $BlueBaseYpos_;
   panMapTo();
}
function clientCmdSetRedPos(%x, %y)
{
	$RedBaseXpos_ =  %x;
	$RedBaseYpos_ =  %y;
}
function clientCmdSetBluePos(%x, %y)
{
	$BlueBaseXpos_ = %x;
	$BlueBaseYpos_ = %y;
}


function SetTeamIcon()
  {
   %team = Canvas.connection.teamID;
   TeamIcon.setBitmap("");
   if (%team == 1)  TeamIcon.setBitmap("starter.combat/client/ui/RedTeamInsigniaButton_h.png");
   if (%team == 2)  TeamIcon.setBitmap("starter.combat/client/ui/BlueTeamInsigniaButton_h.png");
  }

// called from the autospawn button.
// check and see if there is an existing spawn point, and if so - use it
function UseExistingSpawnPoint()
  {
   commandToServer('CheckAutoSpawn');
  }

// This function is called from C++ from within GuiCommanderHud.cc
function GuiCommanderHudIconClick(%clientObj)
 {
  commandToServer('GuiCommanderHudIconClick', ServerConnection.GetGhostIndex(%clientObj));
}

// This function is called from C++ from within GuiCommanderHud.cc
function GuiCommanderHudRightClick(%worldPos)
{
  // pop a dialog, find out what they wanna do with the click
  // then call a suitable servercommand and pass the param.
  // %worldPos is a string with x,y,z.  The z param is the
  // height of the terrain at the location which was clicked
  if ( CommanderMapGui.pickSpawn == 0)
    {
     $worldPos = %worldPos;
     Canvas.pushDialog(commandMapRightClickDlg);
    }
}

function CommanderMapDropAmmo()
{
   commandToServer('CommanderDropAmmo', $worldPos);
   Canvas.popDialog(commandMapRightClickDlg);
}
function DropMedical()
{
   commandToServer('CommanderDropMedical', $worldPos);
   Canvas.popDialog(commandMapRightClickDlg);
}
function DropTarget()
{
   commandToServer('CommanderDropTarget', $worldPos);
   Canvas.popDialog(commandMapRightClickDlg);
}
function DropWaypoint()
{
   commandToServer('CommanderDropWaypoint', $worldPos);
   Canvas.popDialog(commandMapRightClickDlg);
}

// Client has moved (or been moved) to a new team.  update and react accordingly
function clientCmdTeamChanged(%newTeamID)
  {
   Canvas.connection.teamID = %newTeamID;
   SetTeamIcon();
   CommanderMapView.updateShowItems();

   // get current radar status
   commandToServer('GetRadarStatus');
  }

// radar status received from server.  update map
function clientCmdRadarStatus(%status)
  {
   %status = detag(%status);
   if ( %status $= "on")
     {
      CommanderMapView.setShowAllTeams();
     }
   if ( %status $= "off")
     {
      CommanderMapView.setShowOnlyOwnTeam();
     }
   CommanderMapView.updateShowItems();
  }
