//-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// PlayGui is the main TSControl through which the game is viewed.
// The PlayGui also contains the hud controls.
//-----------------------------------------------------------------------------

function PlayGui::onWake(%this)
{
   // Turn off any shell sounds...
   // alxStop( ... );

   $enableDirectInput = "1";
   activateDirectInput();

   // just update the action map here
   MoveMap.push();

   if (GenericDriverMap.active)  GenericDriverMap.push();
   if (GenericPassengerMap.active) GenericPassengerMap.push();
   if (heliDriverMap.active) heliDriverMap.push();
   if (heliPassengerMap.active) heliPassengerMap.push();

   if ($ignoreOnAwake == 1)
     {
      $ignoreOnAwake = 0;
      return;
     }

	playGui_onAwake();
}

function playGui_onAwake()
{

   // Message hud dialog
   Canvas.pushDialog( MainChatHud );
   chatHud.attach(HudMessageVector);


   // hack city - these controls are floating around and need to be clamped
   schedule(0, 0, "refreshCenterTextCtrl");
   schedule(0, 0, "refreshBottomTextCtrl");
}

function PlayGui::onSleep(%this)
{
   Canvas.popDialog( MainChatHud  );

   // pop the keymaps
   moveMap.pop();
}

// reset the visibility of all gui elements back to normal
function PlayGui::resetgui(%this)
  {
   TurretScope.visible=0;
   ScopeGuiGroup1.visible=0;
   ScopeGuiGroup2.visible=0;
   ScopeGuiGroup3.visible=0;
   showPlayerAmmo();
   heliDriverMap::removeGui();
   GenericDriverMap::removeGui();
  }

//-----------------------------------------------------------------------------

function refreshBottomTextCtrl()
{
   BottomPrintText.position = "0 0";
}

function refreshCenterTextCtrl()
{
   CenterPrintText.position = "0 0";
}

function clientCmdSetAmmoAmountHud(%amount, %max, %type)
  {
   $HUDAmmoAmount = (%amount);
   AmmoBar.DrainTo(%amount / %max);
   AmmoAmount.setText(%amount);
  }

function clientCmdSetClipAmountHud(%amount, %max, %type)
  {
   MagazinesBar.DrainTo(%amount / %max);
   ClipAmount.setText(%amount);
  }

function clientCmdSetVehicleAmmo(%amount, %max, %type)
  {
   $HUDAmmoAmount = (%amount);     // GUIs read this value....
   if (isObject(HeliAmmoGauge) && %type $= "RocketLauncherAmmo")
       HeliAmmoGauge.setRocketState(0);
  }

// update team scores.  %scorData is expected to be a TDL of teamID/Score
function ClientCmdSetTeamScores(%scoreData)
  {
   for ( %x=0; %x < getRecordCount(%scoreData); %x++)
    {
     %row = getRecord(%scoreData, %x);
     %teamID = getField(%row, 0);
     %value = getField(%row, 1);

     switch ( %teamID)
      {
       case 1:
         RedScore.setText("Red: " @ %value);
       case 2:
         BlueScore.setText("Blue: " @ %value);

       // add more cases here to support more than 2 teams

       default:
         error("Unknown teamID: " @ %teamID );
      }
    }
  }

// objective has been captured/lost.  update UI with new info
function ClientCmdUpdateObjective(%objective, %teamID)
  {
   %control = "Point" @ %objective;
   switch ( %teamID)
    {
     case 0:
       %control.setBitmap("starter.combat/client/ui/NeutralTeamPoint.png");
     case 1:
       %control.setBitmap("starter.combat/client/ui/RedTeamPoint.png");
     case 2:
       %control.setBitmap("starter.combat/client/ui/BlueTeamPoint.png");
     // add code here to support multiple teams
    }
  }

// hide the player ammo and clip bars -- called when entering a vehicle
function hidePlayerAmmo()
  {
   HealthBar.setVisible(false);
   HealthBarBorder.setVisible(false);
   ammoHud.setVisible(false);
  }

//show the player ammo and clip bars -- called when exiting a vehicle
function showPlayerAmmo()
  {
   showPlayerHealth();
   ammoHud.setVisible(true);
  }

// turn on player health bar
function showPlayerHealth()
  {
   HealthBar.setVisible(true);
   HealthBarBorder.setVisible(true);
  }

function ClientCmdDisplayHitGUI()
{
   HitGui.Visible = true;
   schedule(500, 0, "HideHitGUI");
}

function HideHitGUI()
{
   HitGui.Visible = false;
}

function ClientCmdDisplayHeadHitGUI()
{
   HeadHitGui.Visible = true;
   schedule(500, 0, "HideHeadHitGUI");
}

function HideHeadHitGUI()
{
   HeadHitGui.Visible = false;
}

// folowing function called from C++ in player.cc getCameraTransform
// called when spawn cam has 'joined' the player
function ShowPlayGuiEverything()
{
        PlayGuiEverything.visible = 1;
        playGuiEverything.setVisible(true);
	playGui_onAwake();
}


function clientCmdSetCustomCrosshair(%crosshair)
  {
   if (( %crosshair $= "none") || ( strlen(%crosshair) < 5))
     {
      customCrosshair.visible = 0;
      customCrosshair.bitmap = "none";
     }
   else
     {
      customCrosshair.visible = 1;
      customCrosshair.bitmap = "~/client/ui/" @ %crosshair;
     }
   updateCameraMode();
  }

// sent when the camera swaps between first and 3rd person mode
// hide/show guis as needed.
// $firstPerson = 1 first person cam
// $firstPerson = 0 3rd person cam
function updateCameraMode()
  {
   if (  $firstPerson == 0)
     {
      if ( $ZoomOn == true)
        commandToServer( 'CallZoomFunction', 0 );
      customCrosshair.visible = false;
     }
   if (  $firstPerson == 1)
     {
      if ( customCrosshair.bitmap !$= "none")
        {
         customCrossHair.visible = true;
        }
     }
  }
