// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
//Repair Tool

datablock AudioProfile(RepairToolFireSound)
{
   filename = "~/data/sound/MonkeyWrench.ogg";
   description = AudioClose3d;
   preload = true;
};

datablock ProjectileData(RepairToolProjectile)
{
   projectileShapeName = "~/data/shapes/Rifle/projectile.dts";
   directDamage        = 0;
   explosion           = BulletDebrisExplosion;
   splash              = BulletSplash;

   muzzleVelocity      = 0;
   velInheritFactor    = 1;

   armingDelay         = 0.3;
   lifetime            = 150;
   fadeDelay           = 1500;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;
};

datablock ItemData(RepairTool)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/repairtool.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

   // Dynamic properties defined by the scripts
   pickUpName = "a Repair Tool";
   image = RepairToolImage;
};


//--------------------------------------------------------------------------
// Crossbow image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(RepairToolImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/combat/repairtool.dts";
   emap = true;
   crosshair = "crossHair4.png";
   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   eyeOffset = "0.35 0.87 -0.35";
   customLookAnim = "lookrifle";

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = RepairTool;
   projectile = RepairToolProjectile;
   projectileType = Projectile;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.6;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.2;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = RepairToolFireSound;

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.8;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
};


//-----------------------------------------------------------------------------

function RepairToolImage::onFire(%this, %obj, %slot)
{
    RepairRay(%obj);

    // Get the type of projectile we are gonna fire
    %projectile = %this.projectile;

    // Get the weapons projectile spread and ensure it is never 0
    // (we need some spread direction even if it is extremely tiny)
    %spread = %this.projectileSpread;
    %spread = %spread $= "" ? 0.001 : %spread;
    %spread = %spread <= 0 ? 0.001 : %spread;

    // Determine if we are using a shotgun class weapon or not
    // If we are using a shotgun, set the number of projectiles we are going to fire to 12

        // Get the muzzle vector. This is the dead straight aiming point of the gun
        %vector = %obj.getMuzzleVector(%slot);

        // Get our players velocity. We must ensure that the players velocity is added
        // onto the projectile
        %objectVelocity = %obj.getVelocity();

        // Determine scaled projectile vector. This is still in a straight line as
        // per the default example
        %vector1 = VectorScale(%vector, %projectile.muzzleVelocity);
        %vector2 = VectorScale(%objectVelocity, %projectile.velInheritFactor);
        %velocity = VectorAdd(%vector1,%vector2);

        // Determine our random x, y and z points in our spread circle and create
        // a spread matrix.
        %x = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
        %y = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
        %z = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
        %mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);

        // Alter our projectile vector with our spread matrix
        %velocity = MatrixMulVector(%mat, %velocity);

        // Create our projectile
        %p = new (%this.projectileType)()
        {
            dataBlock = %projectile;
            initialVelocity = %velocity;
            initialPosition = %obj.getMuzzlePoint(%slot);
            sourceObject = %obj;
            sourceSlot = %slot;
            client = %obj.client;
        };

        MissionCleanup.add(%p);

    return %p;

}

function RepairRay(%obj)
{
   %EndPos = %obj.getMuzzlePoint($WeaponSlot);

   InitContainerRadiusSearch(%EndPos, 0.5, $TypeMasks::StaticObjectType | $TypeMasks::VehicleObjectType);

   %rayCast=ContainerSearchNext();
   while (%rayCast != 0 )
     {
      %type = %rayCast.getType();
      if ((%type & $TypeMasks::VehicleObjectType) == $TypeMasks::VehicleObjectType)
        schedule(3000, 0, "Repair", %rayCast);

      if ((%type & $TypeMasks::StaticObjectType) == $TypeMasks::StaticObjectType)
        {
         // for some reason the terrainblock gets picked up as a static object type...
         if (%rayCast.getClassName() !$= "TerrainBlock")
           if (%rayCast.getDatablock().getName() $= "Radar")
             schedule(3000, 0, "Repair", %rayCast, %obj.client);
        }
      %rayCast=ContainerSearchNext();
     }
}

function Repair(%obj, %client)
{
     serverPlay3D(CrossbowReloadSound,%obj.getTransform());
     if (%obj.getDamageLevel() != 0)
       {
        %obj.setDamageLevel(%obj.getDamageLevel() - 250);

        // special case for repairing the radar -  make sure the engineer gets credit
        if (%obj.getDamageLevel() == 0)
          {
           if (%obj.getDatablock().getName() $= "Radar")
             {
              $TManager.recordAction(%client, "EnableRadar", %obj);
             }
          }
       }
}
