// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

// TeamManager
// Made for Starter.Combat starter kit
//
// Functions to manage an arbitrary number of teams
//
// notes:  you MUST CALL createTeam prior to using a team number in your game
//
//   Use the global $TManager when referencing the team manager
//
//  Teams are referenced by teamID, not by name.  so use something like this:
//     $TManager.joinTeam(%client, 1)
//  do not use this:  $TManager.joinTeam(%client, "Red")
//  note also that teamID is not the torque id of the team object.


// create a new team.   team names are hard coded to prevent abuse
// Teams must be created before they can be used
function TeamManager::createTeam(%this, %teamID)
  {
   %this.team[%teamID] = TeamManager::teamData(%teamID);
  }

function TeamManager::initialize(%this)
  {
   // zero out all team data
   for (%x=0; %x < 10; %x++)
     %this.team[%x] = 0;

   // start the score counter
   %this.scoreCounter = %this.schedule(30 * 1000, "updateScoreCounters");
   echo("Started update score counters");

   %this.miscObjectsToUpdate = new SimSet();
  }

// answer back a new team based on ID
function TeamManager::teamData(%teamID)
  {
   %newTeam = new ScriptObject(ManagedTeam);
   %newTeam.teamID = %teamID;
   switch (%teamID)
     {
      case 1:
        %newTeam.name = "Red";
      case 2:
        %newTeam.name = "Blue";
      case 3:
        %newTeam.name = "White";
      case 4:
        %newTeam.name = "Green";

      // add as many names here as you want

      default:
        %newTeam.name = "Team Number " @ %teamID;
     }
   %newTeam.score = 0;
   %newTeam.members = new SimSet();
   %newTeam.objectives = new SimSet();
   %newTeam.radar = 'on';
   return(%newTeam);
  }

// %client joins team
function TeamManager::joinTeam(%this, %client, %teamID)
  {
   %team = %this.team[%teamID];
   if ( isObject(%team))
     {
      if (%client.TeamID == %teamID)
        return; // already on team - do nothing

      // client already on some team - leave it.
      if (isObject( %this.team[%client.teamID] ) )
        %this.LeaveTeam(%client);

      %client.TeamID = %teamID;
      %team.members.add(%client);
      %client.team = %team;

      //echo("client " @ %client @ " joins " @ %team.name);
      commandToClient(%client, 'setTeamScores', %this.getScoreData() );
      commandToClient(%client, 'TeamChanged', %teamID );
      messageAll('MsgTeamActivity', "\c2" @ %client.nameBase @ " Joins the " @ %team.name @ " team");

      // Send the current objective holdings to the new client.
      // hard coded for 2 teams only right now
      for ( %index=1; %index<=2; %index++)
       {
        %team = %this.team[%index];
        for (%x= 0; %x < %team.objectives.getCount(); %x++)
          {
           %obj = %team.objectives.getObject(%x);
           commandToClient(%client, 'UpdateObjective', %obj.objectiveNumber, %obj.teamID );
          }
       }
     }
   else
     error("no such team ID: " @ %teamID @ " For client " @ %client);
  }

// %client Leaves team
function TeamManager::LeaveTeam(%this, %client)
  {
   %team = %this.team[%client.teamID];
   if ( isObject(%team))
     {
      %client.TeamID = "none";
      %client.team =0;
      %client.score =0;
      %team.members.remove(%client);
      if(isObject(%client.player))
         %client.player.setTeam(0);
      messageAll('MsgTeamActivity', "\c2" @ %client.nameBase @ " has left the " @ %team.name @ " team");
     }
   else
     error("no such team ID: " @ %teamID @ " For client " @ %client);
  }

// %client has performed some action on something.  update team scores based on this action
// Adding new actions is just a matter of adding a new case statement near the end of this
// function.  Note that team scores are automaticly sent to the clients at the end of this
//   function, so there is no need to do updates for each individual action
function TeamManager::recordAction(%this, %client, %action, %thing)
  {
   %team = %this.team[%client.teamID];
   if ( !isObject(%team) )
     {
      error("Error recording team action " @ %action);
      error("no such team ID: " @ %teamID @ " For client " @ %client);
      return;
     }

   // handling for all actions goes here
   %msg = "Missing team action message " @ %action;
   switch$ (%action)
    {
     case "setFlagOwner" :
       // sent during mission setup so the team manage is updated correctly and no spam messages
       //  are sent to the players
       %team.objectives.add(%thing);

       // now update the clients
       %count = ClientGroup.getCount();
       for (%x= 0; %x < %count; %x++)
        {
         %recipient = ClientGroup.getObject(%x);
         commandToClient(%recipient, 'UpdateObjective', %thing.objectiveNumber, %thing.teamID );
        }
       return;

     case "FlagUp" :
       // %thing is assumed to be the trigger instance that has been captured
       if ( %client.TeamID == %thing.teamID)
         {
          %team.score++;
          %client.score++;
          %msg = %client.nameBase @ " has Captured " @ %thing.ObjectiveName;
          %team.objectives.add(%thing);
         }
       else
          {
           %team.score--;
           %client.score--;
           %msg = %client.nameBase @ " has Raised the enemy flag ???";
          }

       // Flag has changed hands, send new ownerID to clients
       %count = ClientGroup.getCount();
       for (%x= 0; %x < %count; %x++)
        {
         %recipient = ClientGroup.getObject(%x);
         commandToClient(%recipient, 'UpdateObjective', %thing.objectiveNumber, %thing.teamID );
        }

     case "FlagDown" :
       // %thing is assumed to be the trigger instance thats changing ownership
       if ( %client.TeamID == %thing.teamID)
         {
          %team.score--;
          %client.score--;
          %msg = %client.nameBase @ " Lowered their own flag ???";
         }
       else
         {
          %team.score++;
          %client.score++;
          %otherTeam = %this.team[%thing.TeamID];
          %msg = %otherTeam.name @ " team has lost " @ %thing.objectiveName;
          %otherteam.objectives.remove(%thing);
         }

       // Flag is now neutral, update clients
       %count = ClientGroup.getCount();
       for (%x= 0; %x < %count; %x++)
        {
         %recipient = ClientGroup.getObject(%x);
         commandToClient(%recipient, 'UpdateObjective', %thing.ObjectiveNumber, 0);
        }

     case "Kill" :
        // %thing is assumed to be the client of whoever got killed

       if ( %client == %thing)
         {
          %team.score--; // killed self
          %client.score--;
          %client.deaths++;
          %msg = %client.nameBase @ " killed his own dumb self";
         }
       else
         if ( %client.teamID == %thing.teamID)
           {
            %client.score--;
            %client.kills++;
            %thing.deaths++;
            %msg = %client.nameBase @ " killed team member " @ %thing.nameBase;
           }
         else
           {
            %client.score++;
            %client.kills++;
            %thing.deaths++;
            %msg = %client.nameBase @ " killed " @ %thing.nameBase;
           }

     case "chatMessage":
       // send message to all teammates
       for (%x=0; %x < %team.members.getCount(); %x++)
         {
          %member = %team.members.getObject(%x);
          messageClient(%member, 'TeamMessage', "\c3" @ %client.namebase @ ": " @ %thing);
         }

       // and return.  no score updates are done
       return;

     // disabled a radar
     case "DisableRadar" :
       if ( %client.TeamID == %thing.teamID)
         {
          %team.score--;
          %client.score--;
          %msg = %client.nameBase @ " Disabled own radar";
          %team.radar = 'off';
         }
       else
         {
          %team.score++;
          %client.score++;
          %otherTeam = %this.team[%thing.TeamID];
          %msg = %client.nameBase @ " has disabled the " @ %otherTeam.name @ " Radar";
          %otherTeam.radar = 'off';
         }

     // Radar has been repaired
     case "EnableRadar" :
       if ( %client.TeamID == %thing.teamID)
         {
          %team.score++;
          %client.score++;
          %msg = %client.nameBase @ " has repaired the radar";
          %team.radar = 'on';
         }
       else
         {
          %team.score--;
          %client.score--;
          %otherTeam = %this.team[%thing.TeamID];
          %msg = %client.nameBase @ " has repaired the " @ %otherTeam.name @ " Radar";
          %otherTeam.radar = 'on';
         }
     case "PanelDestroyed":
       // %thing is assumed to be the trigger instance thats changing ownership
       if ( %client.TeamID == %thing.teamID)
         {
          %team.score-=300;
          %client.score-=300;
          %msg = %client.nameBase @ " Destroyed their own panel!!";
         }
       else
         {
          %team.score+=300;
          %client.score+=300;
          %otherTeam = %this.team[%thing.TeamID];
          %msg = %client.namebase @ " Has destroyed " @ %otherTeam.name @ " Control panel!";
         }

     default:
       error("Need to extend TeamManager::recordAction to handle new action");
       error(%msg);
    }

   // send team score updates to everyone
   %this.updateScores();

   // tell everybody what happened
   messageAll('MsgTeamActivity', %msg);
  }

// send the chat message to all team members of %client
function ServerCmdTeamChat(%client, %message)
  {
   $TManager.recordAction(%client, "chatMessage", %message);
  }

// send updated team scores to all clients
function TeamManager::updateScores(%this)
  {
   %scoreData = %this.getScoreData();
   %count = ClientGroup.getCount();
   for (%cl= 0; %cl < %count; %cl++)
    {
     %recipient = ClientGroup.getObject(%cl);
     commandToClient(%recipient, 'SetTeamScores', %scoreData);
   }
  }

// return a TDL list of team scores to be sent to the client.
function TeamManager::getScoreData(%this)
  {
   %count =0;
   // start at teamID 1, go up to 10.
   for ( %x=1; %x < 10; %x++)
     {
      %team = %this.team[%x];
      if ( isObject(%team) )
        {
         %row = %team.teamID TAB %team.score;
         %scoreData = setRecord(%scoreData, %count, %row);
         %count++;
        }
     }
   return %scoreData;
  }

// return the team object associated with %teamID or 0 if no such team
function TeamManager::getTeam(%this, %teamID)
  {
   %team = %this.team[%teamID];
   if (isObject(%team) )
     return %team;

   return 0;
  }

// %client has requested team details on %teamID (usually for displaying on the score gui)
// %send back a series of records..
function serverCmdGetTeamdetails(%client, %teamID)
  {
   %team = $TManager.getTeam(%teamID);
   if (%team == 0)
     {
      error("getTeamdetails - invalid teamid: " @ %teamID);
      return;  // invalid teamid.  ignore request
     }

   %string = setRecord(%string, 0, %team.name TAB %team.score);
   for (%x=0; %x < %team.members.getCount(); %x++)
     {
      %obj = %team.members.getObject(%x);
      %string = setRecord(%string, %x+1, %obj.nameBase TAB %obj.score TAB %obj.kills TAB %obj.deaths);
     }
   commandToClient(%client, 'Teamdetails', %string, %teamID);
  }

// return the radar status (on/off) for the clients team
function serverCmdGetRadarStatus(%client)
  {
   %team = $TManager.getTeam(%client.team.teamID);
   if ( %team == 0)
     {
      error("getRadarStatus - invalid team for client: " @ %team);
      return;
     }
   commandToClient(%client, 'radarStatus', %team.radar);
  }

// runs every 30 sec to update team scores.  1 point per flag held
// This also checks to see if its time to end the game
function TeamManager::updateScoreCounters(%this)
  {
   //echo("TeamManager::updateScoreCounters " @ %this);
   %gameEnd = false;
   for ( %id; %id < 5; %id++)
     {
      %team = %this.getTeam(%id);
      if ( %team > 0)
        {
         %team.score += %team.objectives.getCount();
         if ( %team.score >= $Game::EndGameScore)
           {
            %gameEnd = true;
           }
        }
     }
   %this.updateScores();
   if ( %gameEnd)
     {
      // send player scores to the clients
      for ( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ )
        {
         %cl = ClientGroup.getObject( %clientIndex );
         serverCmdGetTeamdetails(%cl, 1);
         serverCmdGetTeamdetails(%cl, 2);
        }
      cycleGame();
      return;
     }

   %this.scoreCounter = %this.schedule(30 * 1000, "updateScoreCounters");
  }


// when a new client joins, we need to update the teamID on all revelant objects
//   Namely spawnspheres and flagpoles.
function UpdateTeamToClient(%client)
  {
   for ( %x=1; %x <= 2; %x++)
     {
      %team = $TManager.getTeam(%x);
      for ( %o=0; %o < %team.objectives.getCount(); %o++)
        {
         %obj = %team.objectives.getObject(%o);
         sendObjectTeamUpdateToClient(%client, %obj.flagpole, %x);

         // now for each objective, update the spawn points
         %group = %obj.flagpole.getGroup();
         for ( %s=0; %s < %group.getCount(); %s++)
           {
            %newObj = %group.getObject(%s);
            if ((%newObj.getClassName() $= "Trigger") || (%newObj.getClassName() $= "SpawnSphere"))
              {
               sendObjectTeamUpdateToClient(%client, %newObj, %x);
              }
           }
        }
     }
   // "other" objects that need to have a team set when a client joins
   for (%x=0; %x < $TManager.miscObjectsToUpdate.getCount(); %x++)
     {
      %obj = $TManager.miscObjectsToUpdate.getObject(%x);
      sendObjectTeamUpdateToClient(%client, %obj, %obj.getTeam());
     }
  }

// add an object to have its teamID updated when a client joins
function TeamManager::addMisObject(%this, %obj)
  {
   %this.miscObjectsToUpdate.add(%obj);
  }

// Setup the team manager global
$TManager = new ScriptObject(TeamManager);
