// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
//Ammo Drop - parachute drops from commander map

//------------------------------------------------------------------------------


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the CrossbowImage is used.

datablock ItemData(AmmoDrop)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/supplycrate.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

   // Dynamic properties defined by the scripts
   pickUpName = "a Ammo Drop";
   throwable=1;
   iconType = $AmmoIcon;
};

function AmmoDrop::onAdd(%this, %obj)
  {
   %obj.repairAmount = 20;
   %obj.maxRepairAmount = 300;
   %obj.currentRepairAmount = 0;
   schedule(3000,0,"DropAmmo",%obj);
  }

function DropAmmo(%obj)
{
   if (!isObject(%obj) ) return; // obj has been deleted, stop here

   %position = %obj.getTransform();
   InitContainerRadiusSearch(%position, 3, $TypeMasks::PlayerObjectType);
   while ((%targetObject = containerSearchNext()) != 0)
   {
      //use this line for same team only ammo
      //if (%targetObject.getState() !$= "Dead" && %targetObject.getTeam() == %obj.getTeam())
      if (%targetObject.getState() !$= "Dead")
      {
         reloadDropTarget(%targetObject);
         serverPlay3D(CrossbowReloadSound,%obj.getTransform());
         %obj.currentRepairAmount += %obj.repairAmount;

         // check for empty weapon.
         %weapon = %targetObject.getMountedImage($WeaponSlot);
         if ( %targetObject.getInventory( %weapon.getName().ammo ) == 0)
           {
            reloadBullets(%weapon, %targetObject);
           }
      }
   }

   if(%obj.currentRepairAmount >= %obj.maxRepairAmount)
      %obj.delete();
   else
      schedule(3000, 0, "DropAmmo",%obj);
}

function reloadDropTarget(%col)
{
      if(%col.maxInventory(GrenadeAmmo) != 0)
      {
         %col.incInventory(GrenadeAmmo,1);
      }
      if(%col.maxInventory(PistolAmmoClip) != 0)
      {
         %col.incInventory(PistolAmmoClip,1);
      }
      if(%col.maxInventory(ShotgunAmmoClip) != 0)
      {
         %col.incInventory(ShotgunAmmoClip,1);
      }
      if(%col.maxInventory(SMGAmmoClip) != 0)
      {
         %col.incInventory(SMGAmmoClip,1);
      }
      if(%col.maxInventory(MachineGunAmmoClip) != 0)
      {
         %col.incInventory(MachineGunAmmoClip,1);
      }
      if(%col.maxInventory(RifleAmmoClip) != 0)
      {
         %col.incInventory(RifleAmmoClip,1);
      }
      if(%col.maxInventory(AssaultRifleAmmoClip) != 0)
      {
         %col.incInventory(AssaultRifleAmmoClip,1);
      }
      if(%col.maxInventory(SniperRifleAmmoClip) != 0)
      {
         %col.incInventory(SniperRifleAmmoClip,1);
      }
      if(%col.maxInventory(AssaultGrenadeAmmo) != 0)
      {
         %col.incInventory(AssaultGrenadeAmmo,1);
      }
      if(%col.maxInventory(RocketLauncherAmmo) != 0)
      {
         %col.incInventory(RocketLauncherAmmo,1);
      }
      //if(%col.maxInventory(APMineAmmo) != 0)
      //{
      //   %col.incInventory(APMineAmmo,1);
      //}
      //if(%col.maxInventory(AVMineAmmo) != 0)
      //{
      //   %col.incInventory(AVMineAmmo,1);
      //}
      if(%col.maxInventory(MedicPackAmmo) != 0)
      {
         %col.incInventory(MedicPackAmmo,1);
      }
      if(%col.maxInventory(C4Ammo) != 0)
      {
         %col.incInventory(C4Ammo,1);
      }
      if(%col.maxInventory(AmmoPackAmmo) != 0)
      {
         %col.incInventory(AmmoPackAmmo,1);
      }
}

function AmmoDrop::onChuteOpen(%this, %obj)
{
  %obj.onChuteOpen();
}

function AmmoDrop::onChuteClose(%this, %obj)
{
  %obj.onChuteClose();
}
