// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
//Ammo Pack

datablock AudioProfile(AmmoPackReloadSound)
{
filename = "~/data/sound/Ammo_pickup.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(AmmoPackFireSound)
{
filename = "~/data/sound/Ammo_pickup.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(AmmoPackFireEmptySound)
{
filename = "~/data/sound/crossbow_firing_empty.ogg";
description = AudioClose3d;
preload = true;
};

//-----------------------------------------------------------------------------
// Ammo Item

datablock ItemData(AmmoPackAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "Ammo Pack Supplies";
   maxInventory = 10;
};

//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the CrossbowImage is used.

datablock ItemData(AmmoPack)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

   // Dynamic properties defined by the scripts
   pickUpName = "an Ammo Pack";
   image = AmmoPackImage;
   iconType = $AmmoIcon;
   maxInventory = 1;
};


//--------------------------------------------------------------------------
// Crossbow image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(AmmoPackImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/combat/ammo.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   eyeOffset = "0.1 0.4 -0.6";

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = AmmoPack;
   ammo = AmmoPackAmmo;
   //projectile = not used
   //projectileType = not used

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.6;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.2;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = AmmoPackFireSound;

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.8;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateSound[4]                    = AmmoPackReloadSound;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";
   stateScript[5]                   = "onNoAmmo";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   stateSound[6]                    = AmmoPackFireEmptySound;
};


//-----------------------------------------------------------------------------

function AmmoPackImage::onFire(%this, %obj, %slot)
  {
   %obj.decInventory(%this.ammo,1);
   %obj.throw(AmmoPatch,1);
  }

function AmmoPackImage::onNoAmmo(%this, %obj, %slot)
{
   %obj.changeweapon(%slot + 1);
}


function AmmoPack::onUse(%this,%user)
{
   AmmoPackReload(%user);
}

function AmmoPackReload(%obj)
{
   if(%obj.getMountedImage($WeaponSlot).item $= "AmmoPack" && %obj.getState() !$= "Dead")
   {
     if(%obj.getDamageLevel() != 0)
     {
        serverPlay3D(HealthUseSound,%obj.getTransform());
     }
     schedule(10000, 0, "AmmoPackReload", %obj);
   }
}

//-----------------------------------------------------------------------------
// Ammo Patch items cannot be picked up and are not meant to be added to
// inventory.  Health is applied automatically when an objects collides
// with a patch.
//-----------------------------------------------------------------------------

datablock ItemData(AmmoPatch)
{
   // Mission editor category, this datablock will show up in the
   // specified category under the "shapes" root category.
   category = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/ammo.dts";
   mass = 1;
   friction = 1;
   elasticity = 0.3;
   emap = true;
   maxInventory = 0; // No pickup
   throwable ="1";
};


function AmmoPatch::onCollision(%this,%obj,%col)
  {
   if(%col.getTeam() != %obj.getTeam() ) return;

   if (%col.client)
     messageClient(%col.client, 'MsgAmmoPackUsed', '\c2Ammo Pack Applied');

   serverPlay3D(HealthUseSound,%obj.getTransform());
   %obj.delete();
   reloadTarget(%col);
  }


function reloadTarget(%col)
{
      if(%col.maxInventory(GrenadeAmmo) != 0)
      {
         %col.setInventory(GrenadeAmmo,%col.maxInventory(GrenadeAmmo));
      }
      if(%col.maxInventory(PistolAmmoClip) != 0)
      {
         %col.setInventory(PistolAmmoClip,%col.maxInventory(PistolAmmoClip));
      }
      if(%col.maxInventory(RifleAmmoClip) != 0)
      {
         %col.setInventory(RifleAmmoClip,%col.maxInventory(RifleAmmoClip));
      }
      if(%col.maxInventory(SMGAmmoClip) != 0)
      {
         %col.setInventory(SMGAmmoClip,%col.maxInventory(SMGAmmoClip));
      }
      if(%col.maxInventory(ShotgunAmmoClip) != 0)
      {
         %col.setInventory(ShotgunAmmoClip,%col.maxInventory(ShotgunAmmoClip));
      }
      if(%col.maxInventory(MachineGunAmmoClip) != 0)
      {
         %col.setInventory(MachineGunAmmoClip,%col.maxInventory(MachineGunAmmoClip));
      }
      if(%col.maxInventory(SniperRifleAmmoClip) != 0)
      {
         %col.setInventory(SniperRifleAmmoClip,%col.maxInventory(SniperAmmoClip));
      }
      if(%col.maxInventory(AssaultRifleAmmoClip) != 0)
      {
         %col.setInventory(AssaultRifleAmmoClip,%col.maxInventory(AssaultRifleAmmoClip));
      }
      if(%col.maxInventory(AssaultGrenadeAmmo) != 0)
      {
         %col.setInventory(AssaultGrenadeAmmo,%col.maxInventory(AssaultGrenadeAmmo));
      }
      if(%col.maxInventory(RocketLauncherAmmo) != 0)
      {
         %col.setInventory(RocketLauncherAmmo,%col.maxInventory(RocketLauncherAmmo));
      }
      if(%col.maxInventory(MedicPackAmmo) != 0)
      {
         %col.setInventory(MedicPackAmmo,%col.maxInventory(MedicPackAmmo));
      }
      if(%col.maxInventory(C4Ammo) != 0)
      {
         %col.setInventory(C4Ammo,%col.maxInventory(C4Ammo));
      }
}

