//-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

// Area Trigger
// Author: Michael Billmeyer
//     Extensive Modifications by Jon Paynter,
//
// Place in server/scripts directory
// Place gameUtils.cs in the server/scripts directory
//
// Add a object to be captured in the world editor
//    set the property "DynamicFunction"  to "AreaTriggerInit"
//    set the property "ObjectiveName"  to an english name for the objective:  ie: "Red Base"
//    set the property "objectiveNumber" to some number > 0 -- these should be unique across the mission file
//
// SpawnPoints
//    Spawnpoints must be inside a group.  To add a new spawnpoint to the red base or blue base, create
//    a spawnSphere object in the world editor, place it wherever you want.  if its an arial location, set
//    the datablock to ParachuteMarker.  For ground spawns, set the datablock to SpawnSphereMarker.
//
//    Once the spawnSphere is placed, add it to the SimGroup you want - ex: RedBase or "CenterBridgeBase"
//    When the flag in the simGroup is captured, all the spawnpoints in the group turn to match the team
//    that captured the flag

//-----------------------------------------------------------------------------
// Flag Datablocks
//-----------------------------------------------------------------------------

// Team 1 Flag
datablock StaticShapeData( TeamOneFlag )
{
	category = "Flags";
        shapeFile = "~/data/shapes/combat/flag1.dts";
};

// Team 2 Flag
datablock StaticShapeData( TeamTwoFlag )
{
	category = "Flags";
        shapeFile = "~/data/shapes/combat/flag2.dts";
};

// Neutral Flag
datablock StaticShapeData( NeutralFlag )
{
	category = "Flags";
        shapeFile = "~/data/shapes/combat/flagn.dts";
};

// Flag Pole
datablock StaticShapeData( FlagPole )
{
	category = "Flags";
    shapeFile = "~/data/shapes/combat/flagpole.dts";
	iconType = $FlagIcon;
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// This is the trigger
//-----------------------------------------------------------------------------

datablock TriggerData(AreaTrigger)
{
   tickPeriodMS = 100;
};

//-----------------------------------------------------------------------------

function AreaTrigger::onEnterTrigger(%this,%trigger,%obj)
  {
   if ( !canUpdateFlags(%obj))
     {
      return;
     }

   // If both teams are in the zone do nothing.
   if ( ((%this.teamInZone[1] >=1) && (%this.teamInZone[2] >=1)) )
   {
      return;
   }
   else
   {
      if (%obj.client.team.teamID == 1)// team 1 entered zone
         %trigger.teamInZone[1]++;

      if (%obj.client.team.teamID == 2)// team 2 entered zone
         %trigger.teamInZone[2]++;

    }

   Parent::onEnterTrigger(%this,%trigger,%obj);
}

function AreaTrigger::onLeaveTrigger(%this,%trigger,%obj)
{
   if (%obj.client.team.teamID == 1)   // team 1 left zone
   {
      %trigger.teamInZone[1]--;         // decrement teamInZone[1]
      if ( %trigger.teamInZone[1] <= 0) // dont ever go below zero
      {
         %trigger.teamInZone[1] = 0;
         }
     	}
      if (%obj.client.team.teamID == 2)// team 2 left zone
      {
      	%trigger.teamInZone[2]--;
      	if ( %trigger.teamInZone[2] <= 0) // dont ever go below zero
      	{
         	%trigger.teamInZone[2] = 0;
      	}
      }
    Parent::onLeaveTrigger(%this,%trigger,%obj);
    if ((%trigger.teamInZone[1] + %trigger.teamInZone[2]) == 0)
      {
       %trigger.ticks = 0;
      }
}


//---------------------------------------------------------------------------------------------------
// This function gets called every datablock tick and evaluated and changes the trigger ownership
//---------------------------------------------------------------------------------------------------

function AreaTrigger::onTickTrigger(%this,%trigger)
  {
   Parent::onTickTrigger(%this,%trigger);
   %trigger.ticks++;
   if (%trigger.ticks > 1000)
     {
      // somethign has been in the trigger for too many ticks. Re-compute the
      // team standings incase something got left behind in the trigger zone.
      // this prevents unresponsive triggers
      %this.recomputeTeams(%trigger);
     }

   if ( ((%trigger.teamInZone[1] >=1) && (%trigger.teamInZone[2] >=1)) )
     {
      return;
     }
   %flagUpdate =0;

   // loop thru all the objects in the trigger
   for (%i=0; %i < %trigger.getNumObjects(); %i++)
     {
      %obj = %trigger.getObject(%i); //  Set %obj to be the current object that is being used in the for loop.

      // if this thing has a client, then update the flags
      if ( canUpdateFlags(%obj) )
        {
         %team = %obj.client.team.teamID;
         if (%team==1)
           %oppteam = 2;

         if (%team==2)
           %oppteam = 1;

         %flagUpdate = 1;

         if (%trigger.teamFlag[%oppteam] != 0)
           {
            // If the opposing team flag is not 0 then bring it down
            %trigger.teamFlag[%oppteam] = ( %trigger.teamFlag[%oppteam] - %trigger.teamInZone[%team] );

            if ( (%trigger.teamFlag[%oppteam] <= 0) || (%trigger.flagModel == 0) )
              {
                %trigger.teamFlag[%oppteam] = 0;
                $TManager.recordAction(%obj.client, "FlagDown", %trigger);
                %trigger.teamID = 0;
                TurnSpawners(%trigger, 0);
              }
           }
         else
           {
            //  Check if the owning Team  Flag is up. If it is not up, then bring it up 1. (Do this until the flag is up)
            if (%trigger.teamFlag[%team] < 100)   // if team 1 flag is not 100 then bring it up 1
              {
               %trigger.teamFlag[%team] = ( %trigger.teamFlag[%team] + %trigger.teamInZone[%team] );

               //  Dont let the flag variable go above 100.
               if ( %trigger.teamFlag[%team] >= 100)
                 {
                  %trigger.teamFlag[%team] = 100;   // dont ever go above 100
                  // record a flagUp action.  TeamManager will update everyone
                  %trigger.teamID = %team;
                  $TManager.recordAction(%obj.client, "FlagUp", %trigger);
                  TurnSpawners(%trigger, %team);
                 }
              }
           }
        }
     }

  // only update the flag position if something changed
  if (%flagUpdate == 1)
    adjustFlags( %trigger, %team );
}

// return true of %obj should contribute to updating the flag position
function CanUpdateFlags(%obj)
  {
   if ( %obj.client)
     {
      if ( %obj.getState() $= "Dead")  return false;
      if ( %obj.getControllingClient() == 0) return false;
      return true;
     }
   return false;
  }

// recompute the number of (active) objects in the trigger zone
function AreaTrigger::recomputeTeams(%this, %trigger)
  {
   // reset team counters to 0
   %trigger.TeamInZone[1]=0;
   %trigger.TeamInZone[2]=0;

   for (%i=0; %i < %trigger.getNumObjects(); %i++)
     {
      %obj = %trigger.getObject(%i);

      // exclude dead stuff or driver-less vehicles
      if (CanUpdateFlags(%obj))
        {
         if (%obj.client.team.teamID == 1)
           %trigger.teamInZone[1]++;

         if (%obj.client.team.teamID == 2)
           %trigger.teamInZone[2]++;
        }
     }
   %trigger.ticks=0;
   echo("Re-computed team standings for trigger " @ %trigger @ " red: " @ %trigger.teamInZone[1] @ " blue: " @ %trigger.teamInZone[2]);
  }

function adjustFlags( %trigger , %team)
{
	if (%team == 1)  %otherTeam = 2;
	if (%team == 2)  %otherTeam = 1;

   // teams flag is all the way up.  do nothing
   if (%trigger.teamFlag[%team] == 100) return;

   // flag is down... switch to neutral
   if ((%trigger.teamFlag[1] + %trigger.teamFlag[2]) == 0 )
     {
      %oldflag = %trigger.flag[%trigger.flagmodel];
      %newflag = %trigger.flag[0];
      %team = 0;
      %change = 1;
     }

   // flag is moving up, switch to %team flag
   if (( %trigger.teamFlag[%team] > 0) && (%trigger.flagModel != %team) )
     {
      if (%trigger.flagModel == 0)
        %oldFlag = %trigger.flag[0];
      else
        %oldflag = %trigger.flag[%otherTeam];

      %newflag = %trigger.flag[%team];
      %change = 1;
     }

	if (%change == 1)
	  {
		%trigger.flagpole.unmountObject(%oldflag);
      %oldflag.setHidden(true);
		%oldflag.stopThread(0);
		%trigger.flagpole.mountObject(%newflag,0);
      %newflag.setHidden(false);
		%trigger.flagModel = %team;          // set variable flagModel to team
		%trigger.flagpole.setTeam(%team);
		%newflag.playThread(0, "ambient");
	}
	else
	{
		// if team1 flag is visible, and is not up all the way
		%height = ( %trigger.teamFlag[1] + %trigger.teamFlag[2])/100;
        %trigger.flagpole.setThreadPos(0,%height);
	}
}


//---------------------------------------------------------------------------------------------------
// This function gets called when the trigger is created initialized//
// given %obj is a flagpole DTS instance, create a trigger and mounts flag to it
//---------------------------------------------------------------------------------------------------
function AreaTriggerInit(%obj)
{
	if(%obj.getClassName() $= "TSStatic")
	{
	  error("object " @%obj SPC "must be of type StaticShape");
	  return;
    }

   %name = %obj.getName();
   %pos = %name.getWorldBoxCenter();
   %sizex=20; %sizey=20; %sizez=20;

   // optional offset values to position the trigger
   if (%obj.zOffset)
     %z = %obj.zOffset;
   else
     %z =-10;
   if (%obj.xOffset)
     %x = %obj.xOffset;
   else
     %x=0;
   if (%obj.yOffset)
     %y = %obj.yOffset;
   else
     %y=0;

   %trigger = createTriggerBox(%pos, %name.rotation, %sizex, %sizey, %sizez, "AreaTrigger", %x, %y, %z, "CaptureArea");
   %trigger.objectiveNumber = %obj.objectiveNumber;
   %trigger.objectiveName = %obj.objectiveName;

	%trigger.teamInZone[1] = 0;  // total players in zone
	%trigger.teamInZone[2] = 0;  // total players in zone
	%trigger.teamFlag[0] = 0;    // Team 0 flag count 0-100
	%trigger.teamFlag[1] = 0;    // Team 1 flag count 0-100
	%trigger.teamFlag[2] = 0;    // Team 2 flag count 0-100
	%trigger.flagModel = 0;      // Tracks what flag is up or down ( 0=Neutral, 1=Team 1, 2=Team 2 )

	%trigger.flagpole = %obj;
	%obj.trigger = %trigger;
	%obj.PlayThread(0,"flagup");
   %obj.setThreadPos(0,0); //or where ever you want it

	if(%obj.team)
	   %team = %obj.team;
	else
	   %team = 0;

	%trigger.flag[0] = new StaticShape()
	{
		dataBlock = NeutralFlag;
	};
	%trigger.flag[1] = new StaticShape()
	{
		dataBlock = TeamOneFlag;
	};
	%trigger.flag[2] = new StaticShape()
	{
		dataBlock = TeamTwoFlag;
	};
	%trigger.flag[0].setHidden(true);
	%trigger.flag[1].setHidden(true);
	%trigger.flag[2].setHidden(true);

   %trigger.flag[%team].setHidden(false);
	%trigger.flagpole.mountObject(%trigger.flag[%team],0);
	%trigger.flag[%team].playThread(0, "ambient");

}

//set the intial ownership of this flagpole to the red team
function setRedTeamOwner(%obj)
  {
   setInitialOwner(%obj, 1);
  }

//set the intial ownership of this flagpole to the red team
function setBlueTeamOwner(%obj)
  {
   setInitialOwner(%obj, 2);
  }

// setup a flagpole and set its initial owner to %team
// used for setting up the initial flagpoles and spawn points in each base
function setInitialOwner(%obj, %team)
  {
   AreaTriggerInit(%obj);
   %obj.trigger.TeamFlag[%team] = 99;
   adjustFlags(%obj.trigger, %team);
   adjustFlags(%obj.trigger, %team); // call a 2nd time so the flag is positioned correctly

   %obj.trigger.TeamFlag[%team] = 100;
   %obj.trigger.TeamID = %team;

   // update the teamManager so the clients will see this
   %tmpClient = new ScriptObject() { teamID = %team; };
   $TManager.recordAction(%tmpClient, "SetFlagOwner", %obj.trigger);
  }

function TurnSpawners(%trigger, %team)
{
   %flaggroup = %trigger.flagpole.getGroup();

   for (%i = 0; %i < %flaggroup.getCount(); %i++)
     {
      %obj = %flaggroup.getObject(%i);
      if (%obj.getClassName() $= "SpawnSphere")
         %obj.setTeam(%team);

      // change over any ammo drops that were placed in the mission
      if (%obj.getClassName() $= "Item")
        if ( %obj.getDataBlock().getName() $= "AmmoDrop")
          %obj.setTeam(%team);
    }
}

// used to setup fallback  or uncapturable spawnpoints
function SetRedSpawnOwner(%obj)
  {
   %obj.setTeam(1);
   $TManager.addMisObject(%obj);
  }

function SetBlueSpawnOwner(%obj)
  {
   %obj.setTeam(2);
   $TManager.addMisObject(%obj);
  }
