// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
datablock LineTracerData(BulletTracer)
{
   trailLifetimeMS         = 0.250;
   newRingMS               = 0.025;

   numSegments             = 2;

   twistSpeed            = 2880;

   sizes[0] = 0.07;
   sizes[1] = 0.07;
   sizeTimes[0] = 0.0;
   sizeTimes[1] = 1.0;

   colors[0] = "1.00 1.00 1.00 0.45";
   colors[1] = "1.00 0.90 0.50 0.25";//"0.8 0.8 0.8 0.12";
   colors[2] = "1.00 0.70 0.25 0.0";//"0.6 0.6 0.6 0.0";
   times[0] = 0.0;
   times[1] = 0.5;
   times[2] = 1.0;

   texture[0] = "~/data/shapes/particles/spark";

   horizontalTexWrap[0] = 1;
   verticalTexWrap[0] = 1;
   horizontalTexMovementBase[0] = 0;
   verticalTexMovementBase[0] = 0;
};

exec("~/server/scripts/tracer.cs");

// Debris "spark" explosion
datablock ParticleData(BulletDebrisSpark)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoefficient      = 0;
   gravityCoefficient   = 0.0;
   windCoefficient      = 0;
   inheritedVelFactor   = 0.5;
   constantAcceleration = 0.0;
   lifetimeMS           = 500;
   lifetimeVarianceMS   = 50;
   spinRandomMin = -90.0;
   spinRandomMax =  90.0;
   useInvAlpha   = false;

   colors[0]     = "0.6 0.6 0.5 1.0";
   colors[1]     = "0.3 0.3 0.2 1.0";
   colors[2]     = "0 0 0 0.0";

   sizes[0]      = 0.2;
   sizes[1]      = 0.3;
   sizes[2]      = 0.1;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(BulletDebrisSparkEmitter)
{
   ejectionPeriodMS = 20;
   periodVarianceMS = 0;
   ejectionVelocity = 0.5;
   velocityVariance = 0.25;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = false;
   lifetimeMS       = 300;
   particles = "BulletDebrisSpark";
};

datablock ExplosionData(BulletDebrisExplosion)
{
   emitter[0] = BulletDebrisSparkEmitter;

   // Turned off..
   shakeCamera = false;
   impulseRadius = 0;
   lightStartRadius = 0;
   lightEndRadius = 0;
};

// re-useable function to fire a projectile from some weapon using 'spread'
function fireProjectileWithSpread(%this, %obj, %slot)
  {
   // Decrease the weapons ammo on fire.  Automaticly updates the ammo HUD
   %obj.decInventory(%this.ammo,1);

   // Get the type of projectile we are gonna fire
   %projectile = %this.projectile;

   // Get the weapons projectile spread and ensure it is never 0
   // (we need some spread direction even if it is extremely tiny)
   %spread = %this.projectileSpread;
   %spread = %spread $= "" ? 0.001 : %spread;
   %spread = %spread <= 0 ? 0.001 : %spread;

   // Determine if we are using a shotgun class weapon or not
   // If we are using a shotgun, set the number of projectiles we are going to fire to 12
   %shellcount = 1;
   if (%this.isShotGun)
     %shellcount = 12;

   // Create each projectile and send it on its way
   for(%shell=0; %shell<%shellcount; %shell++)
     {
      // Get the muzzle vector. This is the dead straight aiming point of the gun
      %vector = %obj.getMuzzleVector(%slot);

      // Get our players velocity. We must ensure that the players velocity is added
      // onto the projectile
      %objectVelocity = %obj.getVelocity();

      // Determine scaled projectile vector. This is still in a straight line as
      // per the default example
      %vector1 = VectorScale(%vector, %projectile.muzzleVelocity);
      %vector2 = VectorScale(%objectVelocity, %projectile.velInheritFactor);
      %velocity = VectorAdd(%vector1,%vector2);

      // Determine our random x, y and z points in our spread circle and create
      // a spread matrix.
      %x = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
      %y = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
      %z = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
      %mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);

      // Alter our projectile vector with our spread matrix
      %velocity = MatrixMulVector(%mat, %velocity);

      // Create our projectile
      %p = new (%this.projectileType)()
        {
          dataBlock = %projectile;
          initialVelocity = %velocity;
          initialPosition = %obj.getEyePoint();
          sourceObject = %obj;
          sourceSlot = %slot;
          client = %obj.client;
        };

      MissionCleanup.add(%p);
     }

   if (%obj.getInventory(%this.ammo) <= 0)
     {
      %this.ReloadAmmoClip(%obj);
     }
   return %p;
  }

// re-usable function to fire a projectile from some weapon
function fireProjectile(%this, %obj, %slot)
  {
   
   // Decrease the weapons ammo on fire.  Automaticly updates the ammo HUD
   %obj.decInventory(%this.ammo,1);

   // Get the type of projectile we are gonna fire
   %projectile = %this.projectile;

   // Determine if we are using a shotgun class weapon or not
   // If we are using a shotgun, set the number of projectiles we are going to fire to 12
   %shellcount = 1;
   if (%this.isShotGun)
     {
       %shellcount = 12;
     }
   // Create each projectile and send it on its way
   for (%shell=0; %shell<%shellcount; %shell++)
     {
      // Get the muzzle vector. This is the dead straight aiming point of the gun
      %vector = %obj.getMuzzleVector(%slot);

      // Get our players velocity. We must ensure that the players velocity is added
      // onto the projectile
      %objectVelocity = %obj.getVelocity();

      // Determine scaled projectile vector. This is still in a straight line as
      // per the default example
      %vector1 = VectorScale(%vector, %projectile.muzzleVelocity);
      %vector2 = VectorScale(%objectVelocity, %projectile.velInheritFactor);
      %velocity = VectorAdd(%vector1,%vector2);

      // Create our projectile
      %p = new (%this.projectileType)()
        {
         dataBlock = %projectile;
         initialVelocity = %velocity;
         initialPosition = %obj.getEyePoint();
         sourceObject = %obj;
         sourceSlot = %slot;
         client = %obj.client;
         target = %obj.trackTarget;
        };

      %obj.trackTarget =0;
      MissionCleanup.add(%p);
     }

   if (%obj.getInventory(%this.ammo) <= 0)
     {
      %this.ReloadAmmoClip(%obj);
     }

   return %p;
  }

// re-usable function to fire a projectile from some weapon
// this uses the mounted weapon's muzzle offset instead of the eye offset
// not reccomended for player weapons as it affects accuracy
function fireMountedProjectile(%this, %obj, %slot)
  {
   
   // Decrease the weapons ammo on fire.  Automaticly updates the ammo HUD
   %obj.decInventory(%this.ammo,1);

   // Get the type of projectile we are gonna fire
   %projectile = %this.projectile;

   // Determine if we are using a shotgun class weapon or not
   // If we are using a shotgun, set the number of projectiles we are going to fire to 12
   %shellcount = 1;
   if (%this.isShotGun)
     {
       %shellcount = 12;
     }
   // Create each projectile and send it on its way
   for (%shell=0; %shell<%shellcount; %shell++)
     {
      // Get the muzzle vector. This is the dead straight aiming point of the gun
      %vector = %obj.getMuzzleVector(%slot);

      // Get our players velocity. We must ensure that the players velocity is added
      // onto the projectile
      %objectVelocity = %obj.getVelocity();

      // Determine scaled projectile vector. This is still in a straight line as
      // per the default example
      %vector1 = VectorScale(%vector, %projectile.muzzleVelocity);
      %vector2 = VectorScale(%objectVelocity, %projectile.velInheritFactor);
      %velocity = VectorAdd(%vector1,%vector2);

      // Create our projectile
      %p = new (%this.projectileType)()
        {
         dataBlock = %projectile;
         initialVelocity = %velocity;
         initialPosition = %obj.getMuzzlePoint(%slot);
         sourceObject = %obj;
         sourceSlot = %slot;
         client = %obj.client;
         target = %obj.trackTarget;
        };

      %obj.trackTarget =0;
      MissionCleanup.add(%p);
     }

   if (%obj.getInventory(%this.ammo) <= 0)
     {
      %this.ReloadAmmoClip(%obj);
     }

   return %p;
  }


// re-usable reloadammo clip function
function ReloadAmmoClip(%this, %obj, %delay)
  {
   if (%obj.getInventory(%this.clip) > 0)
     {
      %message = "Reloading...";
      %time = 3;
      %lines = 1;

      %obj.playThread(0,"onReload");
      centerPrint( %obj.client, %message, %time, %lines );

      //remember to change the delay to the number of milliseconds
      //that your animation lasts
      schedule(%delay, 0, "reloadBullets",%this,%obj);
     }
   else
     {
      %message = "You are out of Ammo.";
      %time = 3;
      %lines = 1;
      centerPrint( %obj.client, %message, %time, %lines );
      %obj.changeweapon(1);
     }
  }
