// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
// functions for sliding doors

datablock AudioProfile(FrySound)
{
filename = "~/data/sound/combat/electro.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(Powerhum)
{
filename = "~/data/sound/combat/powerhum.ogg";
description = AudioVCloseLooping3d;
preload = true;
};

datablock AudioProfile(Opendoor)
{
filename = "~/data/sound/combat/door_opening.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(Disabled)
{
filename = "~/data/sound/combat/system_alert.ogg";
description = AudioClose3d;
preload = true;
};

datablock StaticShapeData(door){
        category = "Doors";
        //Points to the DTS object change this for your project
        shapeFile = "~/data/shapes/combat/sdoors.dts";
        position = "0 0 0";
        Scale="1 1 1";
        emap = true;
        receiveSunLight = "1";
        maxDamage = 3000;
        destroyedLevel = 3000;
   explosion       = VehicleExplosion;
   debris          = BaseWheeledVehicleDebri;
 };

datablock TriggerData(DoorTrigger)
{
    tickPeriodMS = 500;
};

// called from setting dynamicFunction in mission editor
function MakeRedDoorTrigger(%door)
 {
  MakeDoorTrigger(%door, 1);

 }
function MakeBlueDoorTrigger(%door)
 {
  MakeDoorTrigger(%door, 2);

 }
function MakeNeutralDoorTrigger(%door)
 {
  MakeDoorTrigger(%door, 0);
 }

// play ambient sound for door
function Door::onAdd(%this,%obj,%amount)
  {
   %obj.playAudio(0,Powerhum);
  }

function door::onDestroyed(%data, %obj, %prevState)
  {

   echo("Door destroyed");
   %obj.schedule(1000, "delete");
   %obj.playAudio(0,Disabled);
   // spawn destroyed door
   new TSStatic() {
      position = %obj.position;
      rotation = %obj.rotation;
      shapeName = "~/data/shapes/combat/destdoor.dts";
   };
  }

function door::damage(%this, %obj, %projectile, %pos, %damage, %damageType)
  {
   if (%obj.getDamageLevel() >= %this.destroyedLevel)
     {
      %obj.setDamageState(Destroyed);
      return;
     }
   echo(%obj.getName() @ " takes " @ %damage @ " damage from " @ %projectile.client.nameBase @ " type: "@ %damageType @ " dmgLevel: " @ %obj.getDamageLevel() );
   if (%obj.getTeam() == %projectile.client.team.teamID)
     return; // anti-greifer check

   // check to see if damage bounces off
   if (%this.armorvalue >= %damage) return;

   // subtract armor value from incomming damage & apply the result
   %value = %damage - %this.armorValue;
   %obj.applyDamage(%value);
  }

function door::onCollision(%this, %obj, %col, %vec, %speed)
  {
   %team = %obj.getTeam();
   if (%team ==0) return;

   if (%team != %col.getTeam())
     {
      // door is electrified - zap enemy player
      %col.damage(%obj, %obj.position, 10, "DoorZap");
      %obj.playAudio(0,FrySound);
     }
  }

// create an open/close trigger for the given door
function MakeDoorTrigger(%door, %teamID)
  {
   %name = %door.getName();
   if ( %name $= "")
     {
      error("door object must have a name");
      return;
     }

   //add the trigger
   %pos = %name.getWorldBoxCenter();
   %sizex=4; %sizey=6; %sizez=3;

   %trig = createTriggerBox(%pos, %name.rotation, %sizex, %sizey, %sizez, "DoorTrigger", 0, 0, -3, %name@"Opener");
   %trig.door = %door;
   %door.setTeam(%teamID);
   MissionCleanup.add(%trig);

  }

// open door when players enter the trigger
function DoorTrigger::onEnterTrigger( %this, %trigger, %obj )
  {
   if (%obj.getClassName() !$= "Player")
      return; // only for players

   %door = %trigger.door;
   if ( strlen(%door.state) < 2 ) %door.state= "closed"; // lazy init
   %team = %door.getTeam();
   if (%team != 0)
     {
      // non-neutral doors only open for same team
      if (%team != %obj.getTeam()) return;
     }

  if (%door.state $= "closed")
    {
     %door.state = "open";
     %door.setThreadDir(0, 1);
     %door.playThread(0, "open");
     %obj.playAudio(0,Opendoor);
    }
  }

// close door when players leave the trigger
function DoorTrigger::onLeaveTrigger( %this, %trigger, %obj )
   {
    if (%obj.getClassName() !$= "Player")
      return; // only for players

    %door = %trigger.door;
   %team = %door.getTeam();
   if (%team != 0)
     {
      // non-neutral doors only active for same team
      if (%team != %obj.getTeam()) return;
     }

    if ( %door.state $= "open")
      {
       %obj.playAudio(0,Opendoor);
       %door.setThreadDir(0, 0);
       %door.playThread(0, "open");
       %door.state = "closed";
      }
   }

