 //-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------
// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

// The folowing allocations must match those in guicommandermap.cc
// They define the enumerations for icons
$TankIcon = 1;
$JeepIcon = 2;
$GunshipIcon = 3;
$TransportIcon = 4;
$PersonIcon = 5;
$MediIcon = 6;
$AmmoIcon = 7;
$FlagIcon = 8;
$RadarIcon = 9;
$SpawnIcon = 10;
$WaypointIcon = 11;
$TargetIcon = 12;
$ParachuteIcon = 13;

//The generic classes are: Reconnaissance, Support, Engineer, Medic, Assault, Heavy Assault
$PlayerClass1 = new ScriptObject()
{
    playerClassId = 1;
    name = Reconnaissance;
};

$PlayerClass2 = new ScriptObject()
{
    playerClassId = 2;
    name = Support;
};

$PlayerClass3 = new ScriptObject()
{
    playerClassId = 3;
    name = Engineer;
};

$PlayerClass4 = new ScriptObject()
{
    playerClassId = 4;
    name = Medic;
};

$PlayerClass5 = new ScriptObject()
{
    playerClassId = 5;
    name = Assault;
};

$PlayerClass6 = new ScriptObject()
{
    playerClassId = 6;
    name = HeavyAssault;
};
//Starter.Combat

// Game duration in secs, no limit if the duration is set to 0
$Game::Duration = 0;

// When a client score reaches this value, the game is ended.
$Game::EndGameScore = 300;

// Pause while looking over the end game screen (in secs)
$Game::EndGamePause = 10;

// shots will not damage teammates (friendlyfire=false)
// shots DO damage teammates (friendlyfire= true)
$Game::FriendlyFire = true;

//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function onServerCreated()
{
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "Combat Starter Kit";

   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "Deathmatch";

   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   exec("./audioProfiles.cs");
   exec("./envAudioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs");
   exec("./triggers.cs");
   exec("./inventory.cs");
   exec("./shapeBase.cs");
   exec("./item.cs");
   exec("./environment.cs");
   exec("./staticShape.cs");
   exec("./weapon.cs");
   exec("./radiusDamage.cs");
   exec("./environment.cs");
   exec("./explosions.cs");

   exec("common/server/lightingSystem.cs");


   //Starter.Combat
   exec("./area_trigger.cs");
   exec("./bullets.cs");
   exec("./rocket_launcher.cs");
   exec("./assaultrifle.cs");
   exec("./grenades.cs");
   exec("./shotgun.cs");
   exec("./sniperrifle.cs");
   exec("./rifle.cs");
   exec("./smg.cs");
   exec("./mg.cs");
   exec("./pistol.cs");
   exec("./C4.cs");
   exec("./medicpack.cs");
   exec("./ammopack.cs");
   exec("./medicdrop.cs");
   exec("./ammodrop.cs");
   exec("./repairtool.cs");
   exec("./knife.cs");
   exec("./TeamManager.cs");
   exec("./zoom.cs");
   exec("./parachute.cs");

   exec("./player.cs");

   // Dream ->
   exec("./genericTurret.cs");
   exec("./helivehicle.cs"); // parent, must come before derived heli's
   exec("./heli.cs");
   exec("./vehicle.cs"); // parent, must come before derived vehicles's
   exec("./wheeledvehicle.cs"); // parent, must come before derived wheeledvehicles's
   exec("./jeep.cs");
   exec("./transport.cs");
   exec("./tank01.cs");
   exec("./chaingun.cs");
   exec("./rocketpod.cs");
   exec("./gameUtils.cs");
   exec("./radar.cs");
   exec("./controlpanel.cs");
   exec("./door.cs");

   //Climbable Objects
   exec("./ClimbableObjects.cs");

   //Starter.Combat

   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;

   // starting teams
}

function onServerDestroyed()
{
   // This function is called as part of a server shutdown.
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.
   startGame();
}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed

   // Normally the game should be ended first before the next
   // mission is loaded, this is here in case loadMission has been
   // called directly.  The mission will be ended if the server
   // is destroyed, so we only need to cleanup here.
   cancel($Game::Schedule);
   $Game::Running = false;
   $Game::Cycling = false;
}


//-----------------------------------------------------------------------------

function startGame()
{
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameStart');

      // Other client specific setup..
      %cl.score = 0;
      %cl.teamid=0;
   }

   // Start the game timer
   if ($Game::Duration)
      $Game::Schedule = schedule($Game::Duration * 1000, 0, "onGameDurationEnd" );
   $Game::Running = true;

   $FlagsInitialized = 0;
   TurnSpawners(RedFlagPole.trigger, 1);
   TurnSpawners(BlueFlagPole.trigger, 2);
}

function endGame()
{
   if (!$Game::Running)  {
      error("endGame: No game running!");
      return;
   }

   // Stop any game timers
   cancel($Game::Schedule);

   // Inform the client the game is over
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd');
   }

   // Delete all the temporary mission objects
   resetMission();
   $Game::Running = false;
}

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.
   if ($Game::Duration && !isObject(EditorGui))
      cycleGame();
}


//-----------------------------------------------------------------------------

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      $Game::Cycling = true;
      $Game::Schedule = schedule(0, 0, "onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   $Game::Schedule = schedule($Game::EndGamePause * 1000, 0, "onCyclePauseEnd");
}

function onCyclePauseEnd()
{
   $Game::Cycling = false;

   // Just cycle through the missions for now.
   %search = $Server::MissionFileSpec;
   for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search)) {
      if (%file $= $Server::MissionFile) {
         // Get the next one, back to the first if there
         // is no next.
         %file = findNextFile(%search);
         if (%file $= "")
           %file = findFirstFile(%search);
         break;
      }
   }
   loadMission(%file);
}


//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
function GameConnection::onClientEnterGame(%this)
{
   commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);

   // Create a new camera object.
   %this.camera = new Camera()
   {
      dataBlock = Observer;
   };

   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);

   // Setup game parameters, the onConnect method currently starts
   // everyone with a 0 score.
   %this.score = 0;
   %this.respawn = 0; // tracks whether a player is spawning or not
   %this.team = 0; //default team is 0 (observer)
   %this.playerClass = 0; // default class is 0

   //Spawn a camera by default on a observer point
   %this.camera.setTransform(pickObserverPoint(%this));
   %this.camera.setVelocity("0 0 0");
   %this.setControlObject(%this.camera);
   UpdateTeamToClient(%this);
}

function GameConnection::onClientLeaveGame(%this)
{
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
}


//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%this)
{
   // The control objects invoked this method when they
   // move out of the mission area.
   %this.player.awol = true;
   damageDeserter(%this);
}

function GameConnection::onEnterMissionArea(%this)
{
   // The control objects invoked this method when they
   // move back into the mission area.
   %this.player.awol = false;
}

function damageDeserter(%this)
{
   if(%this.player.awol == true)
   {
      messageClient(%this, 'MsgClientJoin', '\c2Return to the mission area soldier!');
      %pos = %this.player.getPosition();
      %this.player.damage(%this.player, %pos ,20,"Deserter");
      schedule(3000,0, "damageDeserter", %this);
   }
}
//-----------------------------------------------------------------------------

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
  {
   // Clear out the name on the corpse
   %this.player.setShapeName("");
   if (%this.player.mvehicle !=0)
     {
      // we died while in a vehicle. exit the vehicle
      doPlayerDismount(%this, %this.player, 1);
     }
   %this.player.mvehicle = 0;

   // Switch the client over to the death cam and unhook the player object.
   if (isObject(%this.camera) && isObject(%this.player)) {
      %this.camera.setMode("Corpse",%this.player);
      %this.setControlObject(%this.camera);

      schedule(3000, 0, "CommandToClient", %this, 'Respawn');
   }
   %this.player = 0;

   // record the kill
   $TManager.recordAction(%sourceClient, "kill", %this);

}

//-----------------------------------------------------------------------------

function GameConnection::spawnPlayer(%this)
{
   if (%this.respawn == 1)	// If player is spawning then dont spawn again!
     return;

    %this.respawn = 1;

   // Combination create player and drop him somewhere
   if (!%this.Spawner || %this.Spawner.getTeam() != %this.team.teamID)
   {
      %spawn = findAlliedSpawner(%this.team.teamID);
      if (%spawn) %spawnPoint = AdjustSpawn(%spawn);
   }
   else
   {
      %spawn = %this.Spawner;
      %spawnPoint = AdjustSpawn(%spawn) ;
   }

   if(%spawn && %spawn.getDataBlock().getName() $= "ParachuteMarker")
   {
      %this.useChute = 1;
   }

   if(%spawn)
   {
     %this.schedule(0,createPlayer,%spawnPoint); // delay spawn
   }
   else
   {
      Error("this should not be serverside");
      Canvas.pushDialog(ClassSelectDlg);
   }
}

//-----------------------------------------------------------------------------
function GameConnection::createPlayer(%this, %spawnPoint)
{
   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }

   // get the correct datablock for the player. This is the players class.

    // TEAM 1 DATABLOCKS
    if (%this.team.teamId == 1)
    {
        switch ( %this.playerClass.playerClassId )
        {
            // Class 1 - Recon
            case 1:
                %classDatablock = PlayerBody1Class1;  // change to another datablock when we make new models for each class
            // Class 2 - Support
            case 2:
                %classDatablock = PlayerBody1Class2;
            // Class 3 - Engineer
            case 3:
                %classDatablock = PlayerBody1Class3;
            // Class 4 - Medic
            case 4:
                %classDatablock = PlayerBody1Class4;
            // Class 5 - Assault
            case 5:
                %classDatablock = PlayerBody1Class5;
            // Class 6 - Heavy Assault
            case 6:
                %classDatablock = PlayerBody1Class6;
        }
    }

    // TEAM 2 DATABLOCKS
   if (%this.team.teamId == 2)
   {
      switch ( %this.playerClass.playerClassId )
      {
         // Class 1 - Recon
         case 1:
            %classDatablock = PlayerBody2Class1;  // change to another datablock when we make new models for each class
         // Class 2 - Support
         case 2:
            %classDatablock = PlayerBody2Class2;
         // Class 3 - Engineer
         case 3:
            %classDatablock = PlayerBody2Class3;
         // Class 4 - Medic
         case 4:
            %classDatablock = PlayerBody2Class4;
         // Class 5 - Assault
         case 5:
            %classDatablock = PlayerBody2Class5;
         // Class 6 - Heavy Assault
         case 6:
            %classDatablock = PlayerBody2Class6;
        }
   }
   // Create the player object
   %player = new Player() {
      dataBlock = %classDatablock;
      client = %this;
   };


   MissionCleanup.add(%player);

   // Player setup...
   %player.setTransform(%spawnPoint);
   %player.setShapeName(%this.name);

   // Starting equipment for each class
   // calling SetBasicWeaponLoad gives the player a knife, pistol and 4 grenades.


   switch ( %this.playerClass.playerClassId )
     {
      case 1: // Class 1 - Recon
        %player.SetBasicWeaponLoad();

        %player.setInitialInventory(SniperRifleAmmo);
        %player.setInitialInventory(SniperRifleAmmoClip);
        %player.addWeapon(SniperRifle, 3, true);
        %player.mountImage(SniperRifleImage,0);

      case 2: // Class 2 - Support
        %player.SetBasicWeaponLoad();

        %player.setInitialInventory(MachineGunAmmo);
        %player.setInitialInventory(MachineGunAmmoClip);
        %player.setInitialInventory(AmmoPackAmmo);
        %player.addWeapon(MachineGun, 3, true);
        %player.addWeapon(AmmoPack, 5, false);
        %player.mountImage(MachineGunImage,0);

      case 3: // Class 3 - Engineer
        %player.SetBasicWeaponLoad();

        %player.setInitialInventory(ShotgunAmmo);
        %player.setInitialInventory(ShotgunAmmoClip);
        %player.addWeapon(Shotgun, 3, true);
        %player.addWeapon(RepairTool, 5, false);
        %player.mountImage(ShotgunImage,0);

      case 4: // Class 4 - Medic
        %player.SetBasicWeaponLoad();

        %player.setInitialInventory(RifleAmmo);
        %player.setInitialInventory(RifleAmmoClip);
        %player.setInitialInventory(MedicPackAmmo);
        %player.addWeapon(Rifle, 3, true);
        %player.addWeapon(MedicPack, 5, false);
        %player.mountImage(RifleImage,0);

      case 5: // Class 5 - Assault
         %player.SetBasicWeaponLoad();

         %player.setInitialInventory(AssaultRifleAmmo);
         %player.setInitialInventory(AssaultRifleAmmoClip);
         %player.setInitialInventory(AssaultGrenadeAmmo);
         %player.addWeapon(Knife, 1, false);
         %player.addWeapon(Pistol, 2, false);
         %player.addWeapon(AssaultRifle, 3, true);
         %player.mountImage(AssaultRifleImage,0);

      case 6: // Class 6 - Heavy Assault
         // no SetBasicWeaponLoad here because heavy assult doesnt get grenades

         %player.setInitialInventory(PistolAmmo);
         %player.setInitialInventory(PistolAmmoClip);
         %player.setInitialInventory(SMGAmmo);
         %player.setInitialInventory(SMGAmmoClip);

         %player.setInitialInventory(C4Ammo);
         %player.setInitialInventory(RocketLauncherAmmo);
         %player.addWeapon(Knife, 1, false);
         %player.addWeapon(Pistol, 2, false);
         %player.addWeapon(SMG, 3, true);

         %player.addWeapon(C4, 4, false);
         %player.addWeapon(RocketLauncher, 5, true);
         %player.mountImage(RocketLauncherImage,0);
        }


   // Update the camera to start with the player
   %this.camera.setTransform(%player.getEyeTransform());


   // Give the client control of the player
   %this.player = %player;
   %this.setControlObject(%player);
   %this.respawn = 0; // allow player to respawn after dying.
   %player.client = %this;

    %weaponimage = %this.player.getMountedImage(0);
    if (%weaponimage.customLookAnim !$= "")
    {
       %this.player.setArmThread(%weaponimage.customLookAnim);
    }
    else
    {
       %this.player.setArmThread("looknw");
    }

   schedule(0, 0, "sendClientPlayerToClient", %this, %player);

}
function setUpPlayer(%client)
  {
   if (%client.useChute==1)
     %client.player.openChute();

   // send out teamID updates to all existing clients
   %count = ClientGroup.getCount();
   for (%cl= 0; %cl < %count; %cl++)
     {
      %recipient = ClientGroup.getObject(%cl);
      %recipient.player.setTeam(%recipient.team.teamID);
     }
  }

function sendClientPlayerToClient(%client, %player)
{
  %ghost_idx = %client.GetGhostIndex(%player);
  if (%ghost_idx == -1)
  schedule(100, 0, "sendClientPlayerToClient", %client, %player);
  else
     setUpPlayer(%client);
}
//-----------------------------------------------------------------------------
// Support functions
//-----------------------------------------------------------------------------
function AdjustSpawn(%spawner)
  {
   %sphereCenter = %spawner.getTransform();
   %cXoord = getRandom(%spawner.radius)  ;
   %cYoord = getRandom(%spawner.radius)  ;
   %cZoord = getRandom( %spawner.radius) ;
   %newPoint = %cXoord @ " " @  %cYoord @ " " @  %cZoord;
   %randomLocation = MatrixMulPoint(%sphereCenter, %newPoint);
   return %randomLocation;
  }

function pickObserverPoint(%client)
{
   %groupName = "MissionGroup/ObserverDropPoints";
   %group = nameToID(%groupName);

   if (%group != -1) {
      %count = %group.getCount();
      if (%count != 0) {
         %index = getRandom(%count-1);
         %spawn = %group.getObject(%index);
         return %spawn.getTransform();
      }
      else
         error("No observer spawn points found in " @ %groupName);
   }
   else
      error("Missing observer spawn points group " @ %groupName);

   // Could be no spawn points, in which case we'll stick the
   // player at the center of the world.
   return "0 0 300 1 0 0 0";
}

function GameConnection::joinTeam(%this, %teamid)
  {
   $TManager.JoinTeam(%this, %teamID);
  }

function GameConnection::leaveTeam(%client, %teamnumber)
{
   %this.score = 0;

   messageAll('MsgClientLeaveTeam', '\c2%1 has left the %2 team.',
      %client.name,
      %client.team.name,
      %client);

   if (%client.player)
      %client.player.delete();
}

function GameConnection::joinPlayerClass(%this, %classid)
{
   //We only have 6 player classes so if class is greater than 5 or less than 0 its invalid.
   if (%classid > 6 || %classid < 0)
       return false;

   if (%this.respawn == 1)	// If player is spawning then dont spawn again!
     return false;

   if (%classid == 1)
       %this.playerClass = $PlayerClass1;

   if (%classid == 2)
       %this.playerClass = $PlayerClass2;

   if (%classid == 3)
       %this.playerClass = $PlayerClass3;

   if (%classid == 4)
       %this.playerClass = $PlayerClass4;

   if (%classid == 5)
       %this.playerClass = $PlayerClass5;

   if (%classid == 6)
       %this.playerClass = $PlayerClass6;

   %this.spawnPlayer(%this);
}

function findAlliedSpawner(%team)
{
   %groupName = "MissionGroup";
   %group = nameToID(%groupName);

   for (%i = 0; %i < %group.getCount(); %i++)
   {
      %obj = %group.getObject(%i);
      if (%obj.getClassName() $= "SpawnSphere" && %obj.getTeam() == %team)
      {
         %client.Spawner = %obj;
         return %obj;
      }
      else if (%obj.getClassName() $= "SimGroup")
      {
         for (%j = 0; %j < %obj.getCount(); %j++)
         {
            %subobj = %obj.getObject(%j);
            if (%subobj.getClassName() $= "SpawnSphere" && %subobj.getTeam() == %team)
            {
               %client.Spawner = %subobj;
               return %subobj;
            }
         }
      }
   }
}


function sendObjectTeamUpdateToClient(%client, %Obj, %team)
{
  %ghost_idx = %client.GetGhostIndex(%Obj);
  if (%ghost_idx == -1)
  schedule(100, 0, "sendObjectTeamUpdateToClient", %client, %Obj, %team);
  else
    {
     %Obj.setTeam(%team);
    }
}
