// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================


datablock TriggerData(FuelRepairTrigger)
{
    tickPeriodMS = 500;
};

function FuelRepairTrigger::onEnterTrigger( %this, %trigger, %obj )
{
   Parent::onEnterTrigger( %this, %trigger, %obj );
 }

function FuelRepairTrigger::onLeaveTrigger( %this, %trigger, %obj )
{
   Parent::onLeaveTrigger( %this, %trigger, %obj );
}


function FuelRepairTrigger::onTickTrigger( %this, %trigger )
  {
   // You can iterate through the objects in the list by using these
   // methods:
   //    %this.getNumObjects();
   //    %this.getObject(n);

   %Segment = 20; // use this to adjust how fast a repair or refuel occurs

   for (%i = 0; %i < %trigger.getNumObjects(); %i++)
     {
      %obj = %trigger.getObject(%i);
      if (%obj.getDamagePercent() != 1)
        {
      // vehicles only
      if (strstr(%obj.getClassName(), "Vehicle") != -1)
        {
         // Repair the object if needed
         %damage = %obj.getDamageLevel();
         if ( %damage > 0.01 )
           {
            %obj.setDamageLevel( %damage - %Segment );
           }

         // reload ammo
         reloadVehicle(%obj);
         // Note - you may want to add team checks and only repair your own team etc.
        }
       }
     }
  }

// reload ammo that the vehicles carry (tank/jeep/heli)
function ReloadVehicle(%col)
  {
   ReloadThing(%col, ChaingunAmmo, 1);
   ReloadThing(%col, RocketPodAmmo, 1);
   ReloadThing(%col, TankAmmo, 1);

   if (%col.turretNode[1])
     {
      ReloadThing( %col.turretNode[1], CommandTurretAmmo, 2);
      ReloadThing( %col.turretNode[1], TurretAmmo, 2);
     }
   if (%col.turretNode[2])
     {
      ReloadThing( %col.turretNode[2], CommandTurretAmmo, 2);
      ReloadThing( %col.turretNode[2], TurretAmmo, 2);
     }
  }

function ReloadThing(%thing, %ammoName, %quantity)
  {
   if ( %thing.maxInventory(%ammoName) != 0)
      %thing.incInventory(%ammoName, %quantity);
  }

// %obj is a diff to create a heli landing pad with refuel trigger
function createLandingPad(%obj)
  {
   %name = %obj.getName();
   //add the trigger
   %pos = %name.getWorldBoxCenter();
   %sizex=20; %sizey=20; %sizez=15;

   // optional offset values to position the trigger
   if (%obj.zOffset)
     %z = %obj.zOffset;
   else
     %z =0;
   if (%obj.xOffset)
     %x = %obj.xOffset;
   else
     %x=0;
   if (%obj.yOffset)
     %y = %obj.yOffset;
   else
     %y=0;
   if ( %obj.sizex)
      %sizex = %obj.sizex;
   if ( %obj.sizey)
      %sizey = %obj.sizey;
   if ( %obj.sizez)
      %sizez= %obj.sizez;

   %trig = createTriggerBox(%pos, %name.rotation, %sizex, %sizey, %sizez, "FuelRepairTrigger", %x, %y, %z, "FuelArea");

   MissionCleanup.add(%trig);
  }


// %obj is a diff to create a tank repair pad with refuel trigger
function createFuelTrigger(%obj)
  {
   %name = %obj.getName();
   //add the trigger
   %pos = %name.getWorldBoxCenter();
   %sizex=20; %sizey=20; %sizez=8;

   // optional offset values to position the trigger
   if (%obj.zOffset)
     %z = %obj.zOffset;
   else
     %z =0;
   if (%obj.xOffset)
     %x = %obj.xOffset;
   else
     %x=0;
   if (%obj.yOffset)
     %y = %obj.yOffset;
   else
     %y=0;
   if ( %obj.sizex)
      %sizex = %obj.sizex;
   if ( %obj.sizey)
      %sizex = %obj.sizey;
   if ( %obj.sizez)
      %sizex = %obj.sizez;

   %trig = createTriggerBox(%pos, %name.rotation, %sizex, %sizey, %sizez, "FuelRepairTrigger", %x, %y, %z, "FuelArea");

   MissionCleanup.add(%trig);
  }
// above can actually go in its own file.....



function setRedPos(%obj)
  {
   %pos = %obj.getWorldBoxCenter();
   $RedBaseXpos =  getWord(%pos, 0);
   $RedBaseYpos =  getWord(%pos, 1);
  }
function setBluePos(%obj)
  {
   %pos = %obj.getWorldBoxCenter();
   $BlueBaseXpos =  getWord(%pos, 0);
   $BlueBaseYpos =  getWord(%pos, 1);
  }



function createTriggerBox(%pos, %rotation, %scalex, %scaley, %scalez, %dataBlock, %xPosoffset, %yPosoffset, %zPosoffset, %triggerName)
  {
   // triggers are drawn from a corner, not the center of the box
   // you pass in the center of the trigger, angle of rotation,
   // scale and trigger datablock, then this function creates the trigger
   // %xPosoffset and %yPosoffset are optional and are intended as an offset of the %pos
   // prior to drawing the box

   %angle = getWord(%rotation, 3);
   %angleSign = getWord(%rotation, 2);
   if(%angleSign > 0 ) %angle *= -1; //TGE angles seem to run backwards ...

   %xoff_X =   getCos(%angle * 3.14 / 180.0);
   %yoff_X =   getSin(%angle * 3.14 / 180.0);
   %xoff_Y =   getCos((90-%angle) * 3.14 / 180.0);
   %yoff_Y =   getSin((90-%angle) * 3.14 / 180.0);

   // 1st handle the position offsets
   %x = getWord(%pos, 0) + (%xoff_X * %xPosoffset) - (%xoff_Y * %yPosoffset);
   %y = getWord(%pos, 1) + (%yoff_X * %xPosoffset) + (%yoff_Y * %yPosoffset);
   %z = getWord(%pos, 2) + %zPosoffset;
   // now calculate the corner pos i.e. subtract half the scale from x and y
   %x -= (%xoff_X * %scalex/2) + (%xoff_Y * %scaley/2);
   %y -= (%yoff_X * %scalex/2) - (%yoff_Y * %scaley/2);
   %position = %x SPC %y  SPC %z;
   %newTrigger = new Trigger(%triggerName) {
      position = %position;
      rotation = %rotation;
      scale = %scalex SPC %scaley  SPC %scalez;
      dataBlock = %dataBlock;
      polyhedron = "0.0000000 0.0000000 0.0000000 1.0000000 0.0000000 0.0000000 0.0000000 -1.0000000 0.0000000 0.0000000 0.0000000 1.0000000";
      team = 0;
    };
   %newTrigger.empty = true;
   return %newTrigger;
  }


// This is REALLY usefull.  To have a script function run automaticlly when an object
// is loaded, just add a dynamic variable called dynamicFunction to it in the mission file.
// The function name assigned to dynamicFunction will then be executed upon loading
//
function SimGroup::setupDynamicObjects(%this)
  {
   // uncomment these 2 lines for map building, so spurious objects dont get saved to the mission file
   //setRedTeamOwner(RedFlagPole);
   //return;

   %counter = 0;
   for (%i = 0; %i < %this.getCount(); %i++)
     {
      %obj = %this.getObject(%i);
      if (%obj.getClassName() $= "SimGroup")
        %obj.setupDynamicObjects();

      // set 'dynamicFunction'  to the script function to call
      if (%obj.dynamicFunction !$= "")
        {
         %string = %obj.dynamicFunction @ "(" @ %obj @ ");";
         eval(%string);
        }
    }
}


