// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
//Grenades



datablock AudioProfile(FragExplosionSound)
{
filename = "~/data/sound/explosion02_1.ogg";
description = SAudioDefault3d;
preload = true;
};

// Ammo Item

datablock ItemData(GrenadeAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/grenades/frag.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "A Frag Grenade";
   maxInventory = 4;
};


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the GrenadeImage is used.

datablock ItemData(Grenade)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/grenades/frag.dts";
   mass = 0.5;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

	// Dynamic properties defined by the scripts
	pickUpName = "a Frag Grenade";
	image = GrenadeImage;
};


//--------------------------------------------------------------------------
// Grenade image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(GrenadeImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/grenades/frag.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   eyeOffset = "0.35 0.97 -0.35";
   crosshair = "crossHair2.png";
   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";
   customLookAnim = "lookgrenade";
   // Projectile && Ammo.
   item = Grenade;
   ammo = GrenadeAmmo;
   //projectile = not used
   //projectileType = not used

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 1.0;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 1.6;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 1.6;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateScript[4]                 = "onReload";

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateScript[5]                 = "onNoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.6;
   stateTransitionOnTimeout[6]      = "NoAmmo";
};

datablock ProjectileData(GrenadeProjectile)
{
   projectileShapeName = "~/data/shapes/grenades/grenade.dts";
   directDamage        = 100;
   radiusDamage        = 75;
   damageRadius        = 10;
   areaImpulse         = 3000;
   explosion           = GrenadeExplosion;
   splash              = BulletSplash;

   muzzleVelocity      = 0;
   velInheritFactor    = 0.3;

   armingDelay         = 1;
   lifetime            = 9999;
   fadeDelay           = 1;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = true;
   gravityMod = .5;

   hasLight    = true;
   lightRadius = 4;
   lightColor  = "0.8 0.4 0 0.8";

   hasWaterLight     = true;
   waterLightColor   = "0.8 0.4 0 0.8";
};


function GrenadeProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
{
   // Apply damage to the mine all shape base mines
   if (%col.getType() & $TypeMasks::ShapeBasemineType)
   {
      %col.damage(%obj,%pos,%this.directDamage,"GrenadeProjectile","explode");
      commandToClient(%obj.client, 'DisplayHitGUI');
   }

   // Radius damage is a support scripts defined in radiusDamage.cs
   // Push the contact point away from the contact surface slightly
   // along the contact normal to derive the explosion center. -dbs
   radiusDamage(%obj, %pos,%this.damageRadius,%this.radiusDamage,"Radius", %this.areaImpulse);
}


datablock ItemData(GrenadeDevice)
{
   // Mission editor category, this datablock will show up in the
   // specified category under the "shapes" root category.
   category = "Weapons";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/grenadeprojectile.dts";
   mass = 0.5;
   friction = 1;
   elasticity = 0.3;
   emap = true;
   throwOffset = "-1 1 0";

   // Dynamic properties defined by the scripts
   maxInventory = 0; // No pickup or throw
   throwable = "1";
};

function GrenadeImage::onFire(%this, %obj, %slot)
{
   %obj.decInventory(%this.ammo,1);
   %obj.setActionThread("throw");

   %obj.schedule(1300, "Throw", GrenadeDevice,1);

}

function GrenadeImage::onNoAmmo(%this, %obj, %slot)
{
   %obj.changeweapon(%slot + 1);
}

function GrenadeImage::onReload(%this, %obj, %slot)
{
}

function GrenadeDevice::onAdd(%this,%obj)
{
   schedule(5000, 0, "GrenadeDetonate", %obj);
}

function GrenadeDetonate(%obj)
{
   %pos = %obj.getTransform();
   %projectile = new (projectile)()
   {
         dataBlock = GrenadeProjectile;
         initialVelocity  = 0;
         initialposition = %pos;
         client = %obj.client;
    };
    MissionCleanup.add(%projectile);
    %obj.delete();
}


