// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================


//-----------------------------------------------------------------------------
datablock HeliVehicleData(DefaultHeli:HeliVehicleBase)
{
   shapeFile   = "~/data/shapes/heli/helicopter1.dts";
   massCenter = "0 6.1 0";
   // The array numbers below must be sequencial from 0 to numMountPoints -1
   // It does not matter HERE how your mount points are defined in the model
   // check the ::onAdd function to see where you map sequencial seat numbers
   // to your actual numbered mountpoints
   numMountPoints      = 1;        // How many seats or other places to stand?
  // currently uses 0,3,4
   mountPose[0]             = "Sitting";            // Animation name to use in this seat. Point 0 is always the driver...
   mountPointTransform[0]   = "0 0 0 0 0 1 0";      // How far from the node in the model should we be?
   isProtectedMountPoint[0] = true;                // Is this seat safe from outside damage?


   maneuveringForce = 2500;       //Horizontal jets (W,S,D,A key thrust)
   mass = 5500;                   //Mass of the vehicle
   armorValue = 15;
   maxDamage      = 2000.0;      // Used by emitters. Basically, you could think of it as engine damage
   destroyedLevel = 1990.0;      // Used on the object itself, this will call onDestroyed() etc. Hull explosions, client-side


   maxInv[Chaingun]           = 1;
   maxInv[Chaingun2]          = 1;
   maxInv[ChaingunAmmo]       = 10000;
   maxInv[Chaingun2Ammo]      = 10000;
   maxInv[RocketPod]     = 1;
   maxInv[RocketPod2]     = 1;
   maxInv[RocketPodAmmo] = 100;
   IconType = $GunshipIcon;
   minImpactSpeed = 1;       //If hit ground at speed above this then it's an impact
   maxImpactSpeed = 26;   // if hit the grounda above this speed, the heli is destroyed.
   speedDamageScale = 20;
};


//-----------------------------------------------------------------------------

function DefaultHeli::onAdd(%this,%obj)
{
   Parent::onAdd(%this, %obj);

   %obj.setInventory(Chaingun, 1);
   %obj.setInventory(ChaingunAmmo, 10000);
   %obj.mountImage(ChaingunImage, 0);
   %obj.mountImage(Chaingun2Image, 2);

   %obj.setInventory(RocketPod, 1);
   %obj.setInventory(RocketPodAmmo, 100);
   %obj.mountImage(RocketPodImage, 1);
   %obj.mountImage(RocketPod2Image, 3);

   %obj.weaponviews = 0; //used in player.cs onMount

   // The scripting functions expect the player seating mount points
   // to be sequencial from 0 to numMountPoints.  Unfortunately
   // that does not always agree with the actual model layout
   // we thus map the mountpoint in an array
   %obj.seat[0] = 0;
   %obj.seat[1] = 3;
   %obj.seat[2] = 4;

}



// default callback from the engine
// deferr to generic collision function
function DefaultHeli::onCollision(%this, %obj, %col, %vec, %speed)
  {
   doVehicleCollision(%this,%obj,%col,%vec,%speed);
  }

//-----------------------------------------------------------------------------
// Weapon Setup
//-----------------------------------------------------------------------------
function DefaultHeli::onTrigger(%data, %obj, %trigger, %state)
{
   // data = datablock
   // obj = object number
   // trigger = 0 for "fire"
   // state = 1 for firing, 0 for not firing
   if(%trigger == 0)
   {
      switch (%state)
      {
      case 0:
         %obj.fireWeapon = false;
         %obj.setImageTrigger(0, false);
         %obj.setImageTrigger(2, false);
      case 1:
         %obj.fireWeapon = true;
         if(%obj.nextWeaponFire == 2)
         {
            %obj.setImageTrigger(0, true);
            %obj.setImageTrigger(2, false);
         }
         else
         {
            %obj.setImageTrigger(0, false);
            %obj.setImageTrigger(2, true);
         }
      }
   }

   if(%trigger == 1)
   {
      switch (%state)
      {
      case 0:
         %obj.fireWeapon = false;
         %obj.setImageTrigger(3, false);
         %obj.setImageTrigger(4, false);
      case 1:
         %obj.fireWeapon = true;
         if(%obj.nextWeaponFire == 2)
         {
            %obj.setImageTrigger(3, true);
            %obj.setImageTrigger(4, false);
         }
         else
         {
            %obj.setImageTrigger(3, false);
            %obj.setImageTrigger(4, true);
         }
      }
   }

}

// send the initial ammo settings to %client
function DefaultHeli::initialAmmo(%this, %client)
  {
   %currentAmmo = %this.getInventory(ChaingunAmmo);
   %client.setVehicleAmmo(%currentAmmo,%this.dataBlock.maxInv[ChaingunAmmo], "ChaingunAmmo");
  }


// Vehicle specifics upon entering. We always pop other mappings, as this
// function will be called upon changing  seats, as well as upon initial entering.

// These functions are called from helivehicle.cs
function DefaultHeli::onEnterVehicle(%this, %client, %seat)
{
  if (%seat == 0)
  {
     commandToClient(%client, 'CreateHeliVehicleGui', 1, %seat, %this);
     commandToClient(%client, 'PushActionMapHelicopterDriver');
     commandToClient(%client, 'PopActionMapHelicopterPassenger');
  }
  else
  {
     commandToClient(%client, 'CreateHeliVehicleGui', 1, %seat, %this, 0);
     commandToClient(%client, 'PushActionMapHelicopterPassenger');
     commandToClient(%client, 'PopActionMapHelicopterDriver');
  }
  commandToClient(%client, 'PopActionMap');
}

function DefaultHeli::onExitVehicle( %client)
{
  // Shut off all weapons
  %client.player.mVehicle.setImageTrigger(0,0);
  %client.player.mVehicle.setImageTrigger(1,0);
  %client.player.mVehicle.setImageTrigger(2,0);
  %client.player.mVehicle.setImageTrigger(3,0);
  %client.player.mVehicle.setImageTrigger(4,0);

  commandToClient(%client, 'CreateHeliVehicleGui', 0);
  commandToClient(%client, 'PopActionMapHelicopterDriver');
  commandToClient(%client, 'PopActionMapHelicopterPassenger');
  commandToClient(%client, 'PushActionMap');
}
