// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

//-----------------------------------------------------------------------------

// Information extacted from the shape.
//
// Wheel Sequences
//    spring#        Wheel spring motion: time 0 = wheel fully extended,
//                   the hub must be displaced, but not directly animated
//                   as it will be rotated in code.
// Other Sequences
//    steering       Wheel steering: time 0 = full right, 0.5 = center
//    breakLight     Break light, time 0 = off, 1 = breaking
//
// Wheel Nodes
//    hub#           Wheel hub, the hub must be in it's upper position
//                   from which the springs are mounted.
//
// The steering and animation sequences are optional.
// The center of the shape acts as the center of mass for the car.

//-----------------------------------------------------------------------------

datablock ParticleData(CommandTurretFireParticle)
{
   textureName          = "~/data/shapes/turrets/flash02";

   dragCoeffiecient     = 0.0;
   gravityCoefficient   = 0.0;  // no rise
   inheritedVelFactor   = 10.0;

   lifetimeMS           = 60;   // Time in ms
   lifetimeVarianceMS   = 0;    // ...more or less

   spinSpeed            = 15.0;

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1.0 0.0 0.0 0.0";
   colors[1]     = "0.56 0.36 0.26 1.0";
   colors[2]     = "0 0 0 0";

   sizes[0]      = .5;
   sizes[1]      = 1;
   sizes[2]      = .25;

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(CommandTurretFireEmitter)
{
   ejectionPeriodMS = 30;
   periodVarianceMS = 0;

   ejectionVelocity = 0;
   ejectionOffset   = 0;
   velocityVariance = 0;

   thetaMin         = 0.0;
   thetaMax         = 0.0;

   particles = CommandTurretFireParticle;
};

datablock ItemData(CommandTurretAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/crossbow/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "CommandTurret Ammo";
   maxInventory = 300;
};

datablock TurretData( CommandTurret : GenericTurret )
  {
   category                 = "Turrets";
   numPoints                = 100;

   shapeFile                = "~/data/shapes/turrets/commandTurret.dts";
   maxDamage                = 110;
   explosion                = TurretExplosion;
   destroyedLevel           = 110;
   disabledLevel            = 0.84;
   debrisShapeName          = "~/data/shapes/turrets/commandTurret.dts";
   debris                   = TurretDebris;
   renderWhenDestroyed      = false;
   //mountPose[0]             = root;
   numMountPoints           = 1;
   MountPoint               = 0;
   isProtectedMountPoint[0] = true;
   cameraMaxDist            = 20;
   cameraOffset             = 4.5;
   maxEnergy                = 280;
   rechargeRate             = 0.2;
   heat                     = 1.0;


   maxPitchSpeed = 180.0; // barrel up/down degrees/second
   maxYawSpeed   = 180.0;   // rotate degrees/second
   minPitch      = -40; // up angle weaponTray begins to clip yokeBase
   maxPitch      = 30; // down angle handle begins to clip humveecab
   minYaw        = -100;   // we speak of the constraints on yaw as if it's range
   maxYaw        = 100;    // were -180/180 even tho it's actually stored as 0..360

   // mounted by default in TurretData if specified
   barrelSlot               = 0;
   barrel                   = CommandBarrel;

   // crosshair hud image to display when a player controls this turret from a vehicle
   //crosshair = "crossHair5.png";
   crosshair = "none";
   maxInv[CommandTurretAmmo] = CommandTurretAmmo.maxInventory;

  };

datablock ShapeBaseImageData( CommandBarrel : smallTurretGun )
{
   // Basic Item properties
   ammo = CommandTurretAmmo;

// Rex-custom Turret specific values
   stateSequence[1]                 = "Activate";
   stateScript[1]                   = "onActivate";

   stateSequence[2]                 = "Ready";
   stateScript[2]                   = "onReady";

   stateTimeoutValue[3]             = 0.075;
   stateScript[3]                   = "onFire";
   stateEmitter[3]                  = CommandTurretFireEmitter;

   stateTimeoutValue[4]             = 0.075;
   stateSequence[4]                 = "Reload";
   stateScript[4]                   = "onReload";

   stateName[5]                     = "NoAmmo";
   stateTimeoutValue[5]             = 0.1;
   stateTransitionOnAmmo[5]         = "Reload";
   stateTransitionOnTriggerDown[5]  = "DryFire";
   stateSound[5]                    = RifleFireEmptySound;
   stateSequence[5]                 = "NoAmmo";
   stateScript[5]                   = "onNoAmmo";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 0.05;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   stateSound[6]                    = RifleFireEmptySound;
   stateSequence[6]                 = "DryFire";
   stateScript[6]                   = "onDryFire";
};

function CommandTurret::onAdd(%this,%obj)
{
   // if you do not specify a barrel then you
   // need to do this...
   %turretSlot = 1;
   %obj.setEnergyLevel(%this.MaxEnergy);
   %obj.getMountedImage(%turretSlot);
   %obj.mountImage(CommandBarrel,%turretSlot);
   %obj.setInventory(CommandTurretAmmo, CommandTurretAmmo.maxInventory);
}

function CommandBarrel::onFire(%this, %obj, %slot)
  {
   return fireProjectile(%this, %obj, %slot);
  }

function CommandBarrel::ReloadAmmoClip(%this, %obj)
  {
  }

datablock DebrisData(JeepVehicleDebris)
{
    fade = true;
	explodeOnMaxBounce = false;

	elasticity = 0.25;
	friction = 0.10;

	lifetime = 6.5;
	lifetimeVariance = 1.125;

	minSpinSpeed = -90;
	maxSpinSpeed = 90;

	numBounces = 9;
	bounceVariance = 2;

        staticOnMaxBounce = false;
        snapOnMaxBounce = false;
        gravModifier = 2.0;

	useRadiusMass = true;
	baseRadius = 2;

        velocity = 30.0;
	velocityVariance = 12.0;
    terminalVelocity = 60.0;
};


datablock WheeledVehicleTire(JeepVehicleTire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = "~/data/shapes/combat/humveeWheel.dts";
   staticFriction = 2.5;
   kineticFriction = 1.25;

   // Spring that generates lateral tire forces
   lateralForce = 18000;
   lateralDamping = 6000;
   lateralRelaxation = 1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 18000;
   longitudinalDamping = 1800;
   logitudinalRelaxation = 1;
   mass = 75;
   radius = .963;  // seems to do nothing visually, must be tied to math??!?
};

datablock WheeledVehicleSpring(JeepVehicleSpring)
{
   // Wheel suspension properties
   length = 0.65;             // Suspension travel
   force = 1800;              // Spring force
   damping = 360;            // Spring damping
   antiSwayForce = 12;         // Lateral anti-sway force default= 6

};


datablock WheeledVehicleData(Jeep:BaseWheeledVehicle)
{
   category = "Vehicles";
   shapeFile = "~/data/shapes/combat/humvee.dts";
   debrisShapeName = "~/data/shapes/combat/humveeDebris.dts";
   className = JeepData;


   // The array numbers below must be sequencial from 0 to numMountPoints -1
   // It does not matter HERE how your mount points are defined in the model
   // check the ::onAdd function to see where you map sequencial seat numbers
   // to your actual numbered mountpoints

   numMountPoints      = 6;

   mountPose[0]             = "Sitting";
   mountPointTransform[0]   = "0 0 0 0 0 1 0";

   mountPose[1]             = "root";
   mountPointTransform[1]   = "0 0 0 0 0 1 0";

   mountPose[2]             = "Sitting";
   mountPointTransform[2]   = "0 0 0 0 0 1 0";

   mountPose[3]             = "root";
   mountPointTransform[3]   = "0 0 0 0 0 1 0";

   mountPose[4]             = "Sitting";
   mountPointTransform[4]   = "0 0 0 0 0 1 0";

   mountPose[5]             = "Sitting";
   mountPointTransform[5]   = "0 0 0 0 0 1 0";
   // Rigid Body
   mass = 180;
   massCenter = "0 0 0";    // Center of mass for rigid body
   massBox = "0 0 0";         // Size of box used for moment of inertia,

   // Engine
   engineTorque = 370;       // Engine power

   engineBrake = 200;        // Braking when throttle is 0
   brakeTorque = 1000;       // When brakes are applied
   maxWheelSpeed = 90;       // Engine scale by current speed / max speed
   minImpactSpeed = 15;      // Impacts over this invoke the script callback

   maxInv[CommandTurretAmmo]       = 1000;

   // Sounds
   engineSound = JeepEngineSound;

   // Debris datablock
   debris          = JeepVehicleDebris;

   // Starter.Combat values
   maxImpactSpeed = 34;       // if we hit something above this speed, the car is destroyed
   EnglishName = "Jeep";

   damageEmitterOffset[0]  = "0.0 3.8 1.9 ";
   damageEmitterOffset[1]  = "0.0 3.0 2 ";
   iconType = $JeepIcon;
   SpeedDamageScale = 10;
   collisionMass = 180;
};

function Jeep::onAdd(%this,%obj)
{
   Parent::onAdd(%this, %obj);

   // Setup the car with some defaults tires & springs
   for (%i = %obj.getWheelCount() - 1; %i >= 0; %i--) {
      %obj.setWheelTire(%i,JeepVehicleTire);
      %obj.setWheelSpring(%i,JeepVehicleSpring);
      %obj.setWheelPowered(%i,true);    // 4 wheel drive
   }
   // Steer front tires
   %obj.setWheelSteering(0,1);
   %obj.setWheelSteering(1,1);


   %turret = new Turret()
   {
      dataBlock = CommandTurret;
   };


   %obj.mountObject(%turret,6);
   %obj.mountedTurret = %turret;
   // The following command tells player.cs onMount to assign turret control
   // to the person seated at the seat number in the square brackets
   %obj.turretNode[1] = %turret;

   // how many weapons does the driver/pilot control
   %obj.weaponviews = 0; //used in player.cs onMount

   // The scripting functions expect the player seating mount points
   // to be sequencial from 0 to numMountPoints.  Unfortunately
   // that does not always agree with the actual model layout
   // we thus map the mountpoint in an array
   %obj.seat[0] = 0;
   %obj.seat[1] = 1;
   %obj.seat[2] = 2;
   %obj.seat[3] = 3;
   %obj.seat[4] = 4;
   %obj.seat[5] = 5;

   %obj.defaultCam = $eyeView; // see top of vehicle.cs
}

// default callback from the engine
// deferr to generic impact function
function Jeep::onImpact(%this, %obj, %collide, %vec, %vecLen)
 {
  doVehicleImpact(%this, %obj, %collide, %vec, %vecLen);
 }

// Jeep has been destroyed, delete carcas, and schedule a respawn
function Jeep::onDestroyed(%data, %obj, %prevState)
 {
  %obj.schedule($Vehicle::CorpseTimeout, "delete");
  %db = %obj.getDataBlock();
  %pos = %obj.SpawnPos;
  %rot = %obj.SpawnRot;

  if (%db.RespawnTimer)
     Schedule( (%db.RespawnTimer * 1000) + $Vehicle::CorpseTimeout, MissionGroup, "RespawnVehicle", "WheeledVehicle", %db.getName() , %pos, %rot);
 }

// default callback from the engine
// deferr to generic impact function
function Jeep::onCollision(%this, %obj, %col, %vec, %speed)
  {
   doVehicleCollision(%this,%obj,%col,%vec,%speed);
  }

// send the initial ammo settings to %client
function Jeep::initialAmmo(%this, %client, %seat)
{
   %turret = %this.turretNode[%seat];
   if (%turret)
     {
      // get ammo name and max values from whatever turret-like thing is the player is controlling
      %ammo = %turret.dataBlock.barrel.ammo;
      %currentAmmo = %turret.getInventory(%ammo);
      %client.setVehicleAmmo(%currentAmmo, %turret.getDataBlock().maxInv[%ammo], %ammo);
     }
   else
     %client.setVehicleAmmo(0, 0, "none");
}


