// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================
//Knife
datablock AudioProfile(KnifeFireSound)
{
filename = "~/data/sound/combat/knife.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(KnifePullSound)
{
filename = "~/data/sound/takeme.ogg";
description = AudioClose3d;
preload = true;
};

datablock ItemData(Knife)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/combat/combatknife.dts";
   mass = .01;
   elasticity = 0.0;
   friction = 0.0;
   emap = true;

   // Dynamic properties defined by the scripts
   pickUpName = "a Knife";
   image = KnifeImage;
};


//--------------------------------------------------------------------------
// Crossbow image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(KnifeImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/combat/combatknife.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   eyeOffset = "0.38 0.67 -0.45";
   customLookAnim = "lookknife";

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = Knife;
   //projectile = not used
   //projectileType = not used

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 1.6;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 1.2;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = KnifeFireSound;

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.6;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateSound[4]                    = KnifePullSound;
   crosshair="none";  //custom crosshairs -- turn off for the knife

};


//-----------------------------------------------------------------------------

function KnifeImage::onFire(%this, %obj, %slot)
{
     KnifeRay(%obj);
     %obj.setActionThread("r1HandSwing1");
}

function KnifeRay(%obj)
{
   %StartPos=%obj.gettransform();
   %Eye = %obj.getEyeVector();
   %EndPos = vectorScale(%Eye, -1);
   %EndPos = vectorsub(%StartPos,%EndPos);

   InitContainerRadiusSearch(%EndPos, 0.5, $TypeMasks::PlayerObjectType);

   %rayCast=ContainerSearchNext();
   while(%rayCast != 0 && %obj != %rayCast)
   {
      KnifeAttack(%rayCast, %obj);
      %rayCast=ContainerSearchNext();
   }
}

function KnifeAttack(%target, %attacker)
{
     serverPlay3D(CrossbowReloadSound,%attacker.getTransform());
     %target.damage(%attacker, %attacker.getTransform() , 50, KnifeDamage);
}


