// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

datablock AudioProfile(IdleSound)
{
filename = "~/data/sound/combat/radar.ogg";
description = SAudioCloseLooping3d;
preload = true;
};

datablock StaticShapeData(Radar)
{
   category = "Animated Shapes";
   className = DefaultRadarData;
   shapeFile = "~/data/shapes/combat/radar.dts";

   maxDamage = 1000; // destroyed at 1000
   disabledLevel = 800; // disabled beo=yond this damage
   repairRate = 0.05; // repair at 1.6 points per sec


   damageEmitter[0]        = LightDamageEmitter;
   damageEmitter[1]        = HeavyDamageEmitter;
   damageEmitter[2]        = DestroyedEmitter;
   damageEmitterOffset[0]  = "0.0 -3.0 0.0 ";
   damageLevelTolerance[0] = 0.3;
   damageLevelTolerance[1] = 0.7;
   numDmgEmitterAreas      = 1;


	iconType = $RadarIcon;
	armorValue = 949;   // radar is sturdy
	minCollisionSpeed = 1000; // help prevent wierd collision errors
};


function Radar::onAdd(%this,%obj)
{
   %obj.playThread(0,"idle");
   %obj.playAudio(0,IdleSound);
}



function Radar::onDisabled(%data, %obj, %prevState)
  {
   %count = ClientGroup.getCount();
   for (%cl = 0; %cl < %count; %cl++)
     {
       %client = ClientGroup.getObject(%cl);
       if(%client.team.TeamID == %obj.teamID)
            commandToClient(%client, 'radarStatus', 'off');
    }

   %obj.stopThread(0);
   %obj.stopAudio(0);
}

// Radar has been damaged by something.
function Radar::onDamage(%this,%obj,%amount)
{
   %damageAmt = %obj.getDamageLevel();

   if (%damageAmt >= %this.disabledLevel)
     {
      %obj.setDamageState(Disabled);
     }

   // if we get here and amage is zero, then the radar has been repaired
   if (%damageAmt == 0)
     {
      %obj.setDamageState(Enabled);
      %this.onEnable(%obj);
     }
}


// Enable the radar.
// Turn on sound and dish rotation thread
// send commands to the team clients letting to enable the commandermap info
// for the other team.
function Radar::onEnable(%this, %obj)
  {
   %count = ClientGroup.getCount();
   for (%cl = 0; %cl < %count; %cl++)
	{
      %client = ClientGroup.getObject(%cl);
          if (%client.team.TeamID == %obj.teamID)
            commandToClient(%client, 'radarStatus', 'on');

    }
   %obj.playThread(0,"idle");
   %obj.playAudio(0,IdleSound);
  }

// specialized damage function for the radar
// only a C4 charge will hurt it, but a single c4 charge will also disable it.
function Radar::damage(%this,%obj,%sourceObj,%vel,%damage, %damageType)
  {
   // %this = radar getting damaged
   // %obj  = source of the damage (not always filled in)
   // %pos  = position of the damage giver
   // %damage = amount of dmage to do
   // %damageType = type of damage being done

   if (%damageType !$= "C4Projectile") return;

   %db = %this;
   echo(%obj.getClassName() @ " takes " @ %damage @ " damage from " @ %sourceObj.client.getName() @ " type: "@ %damageType);
   if (%obj.getDamageLevel() >= %db.disabledLevel)
     {
      echo(%obj.getName() @ " is disabled - no damage");
      return;
     }

   // if we get here, a c4 charge has landed on the radar and its now disabled.
   %obj.setDamageLevel(%db.disabledLevel);
   %obj.setDamageState(Disabled);

   // Record 'kill' for the radar
   $TManager.recordAction(%sourceObj.client, "DisableRadar", %obj);
  }

// console error spam removal
function StaticShape::getDamageLocation(%pos)
  {
   return "Body";
  }

