// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================


//-----------------------------------------------------------------------------
// Projectile trail emitter
datablock ParticleData(RocketPodSmokeParticle)
{
   textureName          = "~/data/shapes/rocket_launcher/smokeParticle";

   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.1;  // rises
   inheritedVelFactor   = 0.3;

   lifetimeMS           = 200;   // time in ms
   lifetimeVarianceMS   = 50;   // ...more or less

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "0.36 0.36 0.46 1.0";
   colors[1]     = "0.56 0.56 0.56 1.0";
   colors[2]     = "0 0 0 0";

   sizes[0]      = 0.1;
   sizes[1]      = 0.5;
   sizes[2]      = 1.0;

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(RocketPodSmokeEmitter)
{
   ejectionPeriodMS = 30;
   periodVarianceMS = 5;

   ejectionVelocity = 0.25;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 90.0;

   particles = RocketPodSmokeParticle;
};


//-----------------------------------------------------------------------------
// fire emitter

datablock ParticleData(RocketPodFireParticle)
{
   textureName          = "~/data/shapes/rocket_launcher/smokeParticle";

   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.1;  // rises
   inheritedVelFactor   = 0.3;

   lifetimeMS           = 200;   // Time in ms
   lifetimeVarianceMS   = 50;    // ...more or less

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1.0 1.0 1.0 0.0";
   colors[1]     = "0.56 0.36 0.26 1.0";
   colors[2]     = "0 0 0 0";

   sizes[0]      = 0.2;
   sizes[1]      = 0.5;
   sizes[2]      = 1;

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(RocketPodFireEmitter)
{
   ejectionPeriodMS = 30;
   periodVarianceMS = 5;

   ejectionVelocity = 2;
   ejectionOffset   = 0.1;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 10.0;

   particles = RocketPodFireParticle;
};

datablock AudioProfile(RocketPodFireSound)
{
filename = "~/data/sound/combat/rocket.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(RocketPodFireEmptySound)
{
//filename = "~/data/sound/mp5_firing_empty.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(RocketPodExplosionSound)
{
filename = "~/data/sound/combat/explode.ogg";
description = AudioDefault3d;
preload = true;
};

//-----------------------------------------------------------------------------
// When the rocket launcher fires, it will emit a single rocket which behaves
// like this...
//-----------------------------------------------------------------------------
datablock ProjectileData(RocketPodProjectile)
{
	projectileShapeName = "~/data/shapes/rocket_launcher/rocket.dts";

	muzzleVelocity = 200;
	armingDelay    = 0;
	lifetime       = 5000;
	fadeDelay      = 5000;
	isBallistic    = true;
	gravityMod     = 0.5;

    // The projectile will NOT cast light on the ground as it travels.
        hasLight    = false;

    // The rocket will emit exhaust particles using this
	particleEmitter = RocketPodSmokeEmitter;
	//particleEmitter = RocketPodFireEmitter
    // We'll create one of these if our rocket hits something
        explosion    = RocketLauncherExplosion;
        directDamage = 500;
        radiusDamage = 150;
        damageRadius = 40.5;
   areaImpulse  = 1500;
};

function RocketPodProjectile::onCollision( %this, %obj, %col, %fade, %pos, %normal )
{
   // Apply damage to the object all shape base objects
   if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
   {
      %col.damage(%obj,%pos,%this.directDamage,"RocketPod");
      commandToClient(%obj.client, 'DisplayHitGUI');
   }


   // Radius damage is a support scripts defined in radiusDamage.cs
   // Push the contact point away from the contact surface slightly
   // along the contact normal to derive the explosion center. -dbs
   radiusDamage(%obj, %pos, %this.damageRadius, %this.radiusDamage, "Radius", %this.areaImpulse);
}

//-----------------------------------------------------------------------------
// Ammo Item

datablock ItemData(RocketPodAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/crossbow/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

   // Dynamic properties defined by the scripts
   pickUpName = "rocket pod ammo";
   maxInventory = 20;
};

//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the CrossbowImage is used.

datablock ItemData(RocketPod)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The Weapon namespace
   // provides common Weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/rocket_launcher/rocketpod.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = false;
   directUse = 1;
   image = RocketPodImage;

	// Dynamic properties defined by the scripts
   pickUpName = "RocketPod";
};
//-----------------------------------------------------------------------------
// When the player uses the rocket launcher, it will behave like this...
//-----------------------------------------------------------------------------
datablock ShapeBaseImageData(RocketPodImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/rocket_launcher/rocketpod.dts";
   emap = true;
   imageNum = 1;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 3;

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this AssaultRifle doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = RocketPod;
   ammo = RocketPodAmmo;
   projectile = RocketPodProjectile;
   projectileType = Projectile;
   isShotGun = false;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the AssaultRifle is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.50;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the AssaultRifle. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.25;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateEmitter[3]                  = RocketPodFireEmitter;
   stateEmitterTime[3]              = 0.12;
   stateSound[3]                    = RocketPodFireSound;

   // Play the reload animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.5;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateScript[4]                   = "onReload";

   // No ammo in the AssaultRifle, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "onReload";
   stateTransitionOnTriggerDown[5]  = "DryFire";
   stateSound[5]                    = RocketPodFireEmptySound;

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.3;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   fireTimeout = 100;
};

datablock ShapeBaseImageData(RocketPod2Image)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/rocket_launcher/rocketpod.dts";
   emap = true;
   imageNum = 1;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 4;

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this AssaultRifle doesn't actually fire from the muzzle point,
   // we need to turn this off.
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = RocketPod;
   ammo = RocketPodAmmo;
   projectile = RocketPodProjectile;
   projectileType = Projectile;
   isShotGun = false;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the AssaultRifle is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.50;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the AssaultRifle. Calls the fire script which does
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.25;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateEmitter[3]                  = RocketPodFireEmitter;
   stateEmitterTime[3]              = 0.12;
   stateSound[3]                    = RocketPodFireSound;

   // Play the reload animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.5;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateScript[4]                   = "onReload";

   // No ammo in the AssaultRifle, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "onReload";
   stateTransitionOnTriggerDown[5]  = "DryFire";
   stateSound[5]                    = RocketPodFireEmptySound;

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.3;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   fireTimeout = 100;
};

// fire the rocket pod pair as a pair.. using the weapon state timeouts
// of the RocketPodImage
function RocketPodImage::onFire(%this, %obj, %slot)
{
   RocketPodPair::onFire(%this,%obj,%slot);
   %obj.nextWeaponFire = 3;
   schedule(0, 0, "fireNextRocket", %obj);
}

function RocketPod2Image::onFire(%this,%obj,%slot)
{
   RocketPodPair::onFire(%this,%obj,%slot);
   %obj.nextWeaponFire = 1;
}

function RocketPodPair::onFire(%this,%obj,%slot)
  {
   return fireMountedProjectile(%this, %obj, %slot);
  }

function RocketPodImage::onTriggerDown(%this, %obj, %slot)
{
}

function RocketPodImage::onTriggerUp(%this, %obj, %slot)
{
}

function fireNextRocket(%obj)
{
   if(%obj.fireWeapon)
   {
      if(%obj.nextWeaponFire == 0)
      {
         %obj.setImageTrigger(1, true);
         %obj.setImageTrigger(3, false);
      }
         else
         {
            %obj.setImageTrigger(1, false);
            %obj.setImageTrigger(3, true);
         }
   }
            else
            {
               %obj.setImageTrigger(1, false);
               %obj.setImageTrigger(3, false);
            }
}
