//-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

// This file contains ShapeBase methods used by all the derived classes

//-----------------------------------------------------------------------------
// ShapeBase object
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
datablock AudioProfile(Misslelock)
{
filename = "~/data/sound/combat/misslebeep3.ogg";
description = AudioClose3d;
preload = true;
};


function ShapeBase::damage(%this, %sourceObject, %position, %damage, %damageType)
{
   // All damage applied by one object to another should go through this
   // method. This function is provided to allow objects some chance of
   // overriding or processing damage values and types.  As opposed to
   // having weapons call ShapeBase::applyDamage directly.
   // Damage is redirected to the datablock, this is standard proceedure
   // for many built in callbacks.
   %this.getDataBlock().damage(%this, %sourceObject, %position, %damage, %damageType, %damageLoc);
}


//-----------------------------------------------------------------------------

function ShapeBase::setDamageDt(%this, %damageAmount, %damageType)
{
   // This function is used to apply damage over time.  The damage
   // is applied at a fixed rate (50 ms).  Damage could be applied
   // over time using the built in ShapBase C++ repair functions
   // (using a neg. repair), but this has the advantage of going
   // through the normal script channels.
   if (%obj.getState() !$= "Dead") {
      %this.damage(0, "0 0 0", %damageAmount, %damageType);
      %obj.damageSchedule = %obj.schedule(50, "setDamageDt", %damageAmount, %damageType);
   }
   else
      %obj.damageSchedule = "";
}

function ShapeBase::clearDamageDt(%this)
{
   if (%obj.damageSchedule !$= "") {
      cancel(%obj.damageSchedule);
      %obj.damageSchedule = "";
   }
}


//-----------------------------------------------------------------------------
// ShapeBase datablock
//-----------------------------------------------------------------------------

function ShapeBaseData::damage(%this, %obj, %position, %source, %amount, %damageType)
{
   // Ignore damage by default. This empty method is here to
   // avoid console warnings.
}
// Dream Turrets

//-----------------------------------------------------------------------------
// ShapeBaseImage datablock
//-----------------------------------------------------------------------------

// any shape base image does this on fire
function ShapeBaseImageData::onFire(%data, %obj, %slot)
  {
   %projectile = %data.projectile;

   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();

   %muzzleVelocity = VectorAdd(
   VectorScale(%muzzleVector, %projectile.muzzleVelocity),
   VectorScale(%objectVelocity, %projectile.velInheritFactor));

   %p = new (%data.projectileType)()
     {
      dataBlock        = %data.projectile;
      initialVelocity  = %muzzleVelocity;
      initialDirection = %obj.getMuzzleVector(%slot);
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
     };

   if (isObject(%obj.lastProjectile) && %obj.deleteLastProjectile)
      %obj.lastProjectile.delete();

   %obj.lastProjectile = %p;
   %obj.deleteLastProjectile = %data.deleteLastProjectile;

   MissionCleanup.add(%p);

   return %p;
  }

// Dream Turrets

//Rob - Targetting System
// The targetting system performs a ray cast on each valid rocketlauncher image
// every second, and if it locates a vehicle informs them. Ray casts are only
// performed on images which have a controlling client capable of firing.

// This function is called every second for each player mounted launcher that is zoomed in or
// helicopter image which has a pilot. It checks to see if the image still exists,
// and performs some validity checks before we perform a raycast

function RocketLauncherTargetCheck(%obj)
{
	// If our object with the rocket launcher has been deleted since the last call
	// we can't ray cast :)
	if(!isObject(%obj)) {return false;}

	// Check our rocketlauncher wielder is valid! A player or a heli
	if(%obj.getclassname() !$= "player" && %obj.getclassname() !$= "helivehicle"){return false;}

	// If It's a Heli make sure it has a pilot, if not we no longer need to loop
	if(%obj.getclassname() $= "helivehicle" && !(%obj.getMountNodeObject(%vehicle.seat[0]))) {return false;}

	// If it's a player in a vehicle, they don't pose much threat.
	if(%obj.getclassname() $= "Player" && %obj.mVehicle){return false;}

	// Our object is valid, set the flag to say we're still in the loop
	%obj.TrackingLoopActive = true;

	//Perform the ray cast
	%obj.checkForVehicleTargets();
}

// This function is called on threatening rocket launchers performs the actual raycast.
function ShapeBase::CheckForVehicleTargets(%obj)
  {
   // If our object is a player, we're raycasting from him
   if (%obj.getclassname() $= "player")
     {
      %player = %obj;
      %eye = %player.getEyeVector();
     }
   else
     // If our object is a heli, we're raycasting from the pilot. And we
     // can schedule the validity checks again.
     if (%obj.getclassname() $= "helivehicle")
       {
        %player = %obj.getMountNodeObject(%vehicle.seat[0]);

        //Need to make this more accurate
        %eye = %player.getEyeVector();

        //Schedule the loop to happen again in 1 second.
        schedule(1000, 0, RocketLauncherTargetCheck, %obj);
       }

   // with %distance set to 1000, we can target things 1km away
   %distance = 1000;

   // determine where the player is looking, and scale it out by a factor
   // of %distance.  By default, this gives us a 120m long vector
   %vec = vectorScale(%eye, %distance);

   // define the start and end coordinates of the ray
   %startPoint = %player.getEyeTransform();
   %endPoint = VectorAdd(%startPoint,%vec);

   // set the mask to only return a hit on vehicle objects;
   %searchMasks = $TypeMasks::VehicleObjectType | $TypeMasks::VehicleObjectType;

   // cast ray starting at startPoint ending at endPoint with mask, ignoring %player
   %scanTarg = ContainerRayCast (%startPoint, %endPoint, %searchMasks, %player);

   // call the onTargetted function on the nearest vehicle object.
   if(%scanTarg){getWord(%scanTarg, 0).onTargetted(%player);}

}

// set lock-on target for %client
// Lock will stay for 3 seconds.
function setTargetLock(%target, %player)
  {
   if (%client.trackTarget == %target)
     return;
   if (%player.mVehicle == %target)
     return;

   %player.trackTarget = %target;
   %player.playAudio(0,Misslelock);
   schedule(3000, 0, removeTargetLock, %player);
   commandToClient(%player.client, 'SetTargetLock');
  }

// remove target lock for %client
function removeTargetLock(%player)
  {
   %player.trackTarget = 0;
   commandToClient(%player.client, 'RemoveTargetLock');
  }
