// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

//-----------------------------------------------------------------------------
datablock HeliVehicleData(TransHeli:HeliVehicleBase)
{
   shapeFile   = "~/data/shapes/heli/transHeli.dts";
   massCenter = "0 4.6 0";
   // The array numbers below must be sequencial from 0 to numMountPoints -1
   // It does not matter HERE how your mount points are defined in the model
   // check the ::onAdd function to see where you map sequencial seat numbers
   // to your actual numbered mountpoints
   numMountPoints      = 7;        // How many seats or other places to stand?

   mountPose[0]             = "Sitting";            // Animation name to use in this seat. Point 0 is always the driver...
       // How far from the node in the model should we be?
   isProtectedMountPoint[0] = true;                 // Is this seat safe from outside damage?
   mountPointTransform[0]   = "0 0 0 0 0 1 0";
   
   mountPose[1]             = "Sitting";            // Point 1 is always first gunner, if applicable
   mountPointTransform[1]   = "0 0 0 0 0 1 0";                                                  // Is this seat safe from outside damage?
   isProtectedMountPoint[1] = false;
   
   mountPose[2]             = "Sitting";
   isProtectedMountPoint[2] = false;                // Is this seat safe from outside damage?
   mountPointTransform[2]   = "0 0 0 0 0 1 0";
   
   
   mountPose[3]             = "Sitting";            // from here on, all points are passengers (and maybe even the 2 above)
   isProtectedMountPoint[3] = false;                // Is this seat safe from outside damage?
   mountPointTransform[3]   = "0 0 0 0 0 1 0";
   
   mountPose[4]             = "Sitting";// Is this seat safe from outside damage?
   isProtectedMountPoint[4] = false;
   mountPointTransform[4]   = "0 0 0 0 0 1 180";
   
   mountPose[5]             = "Sitting";
   isProtectedMountPoint[5] = false;  
   mountPointTransform[5]   = "0 0 0 0 0 1 180";
                                                   // Is this seat safe from outside damage?
   mountPose[6]             = "Sitting";
   isProtectedMountPoint[6] = false;
   mountPointTransform[6]   = "0 0 0 0 0 1 180";


   // Rigid body
   mass = 6540;                   //Mass of the vehicle
   // Maneuvering
   maneuveringForce = 1600;       //Horizontal jets (W,S,D,A key thrust)
   armorValue = 15;

   maxDamage      = 2950.0;      // Used by emitters. Basically, you could think of it as engine damage
   destroyedLevel = 2950.0;      // Used on the object itself, this will call onDestroyed() etc. Hull explosions, client-side

   // Sounds
   engineSound             = HeliEngineSound;
   // Explosion
   directDamage = 1000;
   damageRadius = 40;
   radiusDamage = 100;
   areaImpulse = 100;

   iconType = $TransportIcon;
   englishName = "Transport Helicopter";
   collisionMass = 168; // 16,864lbs courtesy of google
};


//-----------------------------------------------------------------------------

function TransHeli::onAdd(%this,%obj)
{
   Parent::onAdd(%this, %obj);

   %turret = new Turret()
   {
      dataBlock = GenericAutoTurret;
   };
   %obj.mountObject(%turret,7);
   %obj.mountedTurret[0] = %turret;
   %obj.turretNode[2] = %turret;
   %turret.setRotZ(4.71);
   %turret.setTurretOptions(180,180,2,-175,0,-20,45);

   %turret = new Turret()
   {
      dataBlock = GenericAutoTurret;
   };
   %obj.mountObject(%turret,8);
   %obj.mountedTurret[1] = %turret;
   %obj.turretNode[1] = %turret;
   %turret.setRotZ(1.57);
   %turret.setTurretOptions(180,180,2,10,175,-20,45);
   %turret.setMinYaw(0.174533);         // work around hard coded limits

   // The scripting functions expect the player seating mount points
   // to be sequencial from 0 to numMountPoints.  Unfortunately
   // that does not always agree with the actual model layout
   // we thus map the mountpoint in an array
   %obj.seat[0] = 0;
   %obj.seat[1] = 1;
   %obj.seat[2] = 2;
   %obj.seat[3] = 3;
   %obj.seat[4] = 4;
   %obj.seat[5] = 5;
   %obj.seat[6] = 6;

   %obj.weaponviews = 0; //used in player.cs onMount

}

function TransHeli::onRemove(%this, %obj)
  {
   %pos = %obj.SpawnPos;
   %rot = %obj.SpawnRot;

   if (isObject( %obj.mountedTurret[0]) )
     %obj.mountedTurret[0].delete();

   if (isObject(%obj.mountedTurret[1]) )
      %obj.mountedTurret[1].delete();
}

// default callback from the engine
// deferr to generic collision function
function TransHeli::onCollision(%this, %obj, %col, %vec)
  {
   doVehicleCollision(%this,%obj,%col,%vec,%speed);
  }

//-----------------------------------------------------------------------------
// Weapon Setup
//-----------------------------------------------------------------------------
function TransHeli::onTrigger(%data, %obj, %trigger, %state)
{
   // data = datablock
   // obj = object number
   // trigger = 0 for "fire"
   // state = 1 for firing, 0 for not firing
   if(%trigger == 0)
   {
      switch (%state)
      {
      case 0:
         %obj.fireWeapon = false;
         %obj.setImageTrigger(0, false);
         %obj.setImageTrigger(2, false);
      case 1:
         %obj.fireWeapon = true;
         if(%obj.nextWeaponFire == 2)
         {
            %obj.setImageTrigger(0, true);
            %obj.setImageTrigger(2, false);
         }
         else
         {
            %obj.setImageTrigger(0, false);
            %obj.setImageTrigger(2, true);
         }
      }
   }
}

// send the initial ammo settings to %client
function TransHeli::initialAmmo(%this, %client)
{
   %currentAmmo = %this.getInventory(ChaingunAmmo);
   %client.setVehicleAmmo(%currentAmmo,%this.dataBlock.maxInv[ChaingunAmmo], "ChaingunAmmo");
}


// Vehicle specifics upon entering. We always pop other mappings, as this
// function will be called upon changing  seats, as well as upon initial entering.

// These functions are called from helivehicle.cs
function TransHeli::onEnterVehicle(%this, %client, %seat)
{
  if (%seat == 0)
  {
     commandToClient(%client, 'CreateHeliVehicleGui', 1, %seat, %this);
     commandToClient(%client, 'PushActionMapHelicopterDriver');
     commandToClient(%client, 'PopActionMapHelicopterPassenger');
  }
  else
  {
   if ( %this.turretNode[%seat])
     commandToClient(%client, 'CreateHeliVehicleGui', 1, %seat, %this, 1);
   else
     commandToClient(%client, 'CreateHeliVehicleGui', 1, %seat, %this, 0);

     commandToClient(%client, 'PushActionMapHelicopterPassenger');
     commandToClient(%client, 'PopActionMapHelicopterDriver');
  }
  commandToClient(%client, 'PopActionMap', moveMap);
}

function TransHeli::onExitVehicle( %client)
{
  commandToClient(%client, 'CreateHeliVehicleGui', 0);
  commandToClient(%client, 'PopActionMapHelicopterDriver');
  commandToClient(%client, 'PopActionMapHelicopterPassenger');
  commandToClient(%client, 'PushActionMap', moveMap);
}
