// *******************************************
// vehicle.cs
//
// Copyright (c) 2002 Pixellent Interactive
//

// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

// This script file contains generic code for
// ing and dismounting vehicles, and for
// switching seats.
// *******************************************
datablock AudioProfile(WarningSound)
{
filename = "~/data/sound/combat/warning.ogg";
description = AudioCloseLooping3d;
preload = true;
};


// Define default camera mod options
$eyeView = 1;
$externalView = 0;
$weaponView = 2;

$Vehicle::warningTimer = 3000;  // how much time to play the warning sound
$Vehicle::CorpseTimeout = 75; // how long to keep the destroyed vehicle around Rex- this default[300] seemed a tad too long

function extractSeatFromNode(%vehicle, %node)
{
  %numSeats = %vehicle.getDataBlock().numMountPoints;
  for(%i=0;%i< %numSeats; %i++)
  {
     if(%vehicle.seat[%i] == %node)
       return %i;
  }
}

// *******************************************
// Function: createVehicle
//
// Inputs: Vehicle object
//
// Outputs: None
//
// This function creates a new vehicle using
// the WheeledVehicle datablock.
// *******************************************
function createVehicle(%vehicle)
{
   %car = new WheeledVehicle()
   {
      datablock = %vehicle.dataBlockName;
   };
   %car.mountable = true;
   %car.setEnergyLevel(60);
   %car.setTransform(%vehicle.position);
   MissionCleanup.add(%car);
}
//vehicle explosion

datablock ExplosionData(VehicleExplosion)
{
   soundProfile           = heliExplosionSound;
   lifeTimeMS             = 1200;

   // Volume particles
   particleEmitter        = heliExplosionFireEmitter;
   particleDensity        = 300;
   particleRadius         = 8;

   // Point emission
   emitter[0]             = heliExplosionSmokeEmitter;
   emitter[1]             = heliExplosionSparkEmitter;

   // Sub explosion objects
   subExplosion[0]        = heliSubExplosion1;
   subExplosion[1]        = heliSubExplosion2;

   // Camera Shaking
   shakeCamera            = true;
   camShakeFreq           = "10.0 11.0 10.0";
   camShakeAmp            = "1.0 1.0 1.0";
   camShakeDuration       = 0.5;
   camShakeRadius         = 10.0;

   // Exploding debris
   debris = heliExplosionDebris;
   debrisThetaMin         = 0;
   debrisThetaMax         = 60;
   debrisPhiMin           = 0;
   debrisPhiMax           = 360;
   debrisNum              = 7;
   debrisNumVariance      = 5;
   debrisVelocity         = 0.8;
   debrisVelocityVariance = 0.4;

   // Impulse
   impulseRadius          = 20;
   impulseForce           = 40;

   // Dynamic light
   lightStartRadius       = 6;
   lightEndRadius         = 3;
   lightStartColor        = "0.5 0.5 0";
   lightEndColor          = "0 0 0";
};


// *******************************************
// Function: serverCmdAddCar
//
// Inputs: Client connection object
//
// Outputs: None
//
// This function creates a new WheeledVehicle
// instance near the player when a key is
// pressed. This is testing code, and is no
// longer needed now that vehicles can be
// added to the mission file.
// *******************************************
function serverCmdAddCar(%client)
{
   if (%client.player $= "" )
      return;
   if (%client.player) {
      if (%client.player.isMounted())
         return;
   }
   %car = new WheeledVehicle()
   {
      datablock = DefaultCar;
   };
   %car.mountable = true;
   %car.setEnergyLevel(60);

   // Get the player's position. We will use this to
   // place the car near the player.
   %pos = %client.player.getTransform();
   %x = getWord(%pos, 0);
   %y = getWord(%pos, 1);
   %z = getWord(%pos, 2);
   %x += 5.0;
   %y += 5.0;
   %z += 0.0;
   %car.setTransform(%x SPC %y SPC %z);
   MissionCleanup.add(%car);
}

// *******************************************
// Function: findEmptySeat
//
// Inputs: Vehicle object
//         Vehicle datablock
//
// Outputs: -1 if there are no empty seats
//          The seat number that is free
//
// This function locates the next free seat in
// the vehicle.
// *******************************************


//||||||||||||||||||||||||||||||||||||||||||||||||||||
// called from onMountVehicle in this file
//||||||||||||||||||||||||||||||||||||||||||||||||||||
function findEmptySeat(%vehicle, %vehicleblock)
{
   for (%i = 0; %i <  %vehicleblock.numMountPoints; %i++)
   {
      %node = %vehicle.getMountNodeObject(%vehicle.seat[%i]);
      if (%node == 0)
      {
         return %i;
      }
   }
   return -1;
}
// *******************************************
// Function: onMountVehicle
//
// Inputs: Vehicle object
//         Player datablock
//         Vehicle object
//
// Outputs: None
//
// This function is called by the player's
// onMount function. It locates an empty
// seat in the vehicle and calls the function
// to perform the mount.
// *******************************************

//||||||||||||||||||||||||||||||||||||||||||||||||||||
// called from serverCmdMountVehicle in commands.cs
//||||||||||||||||||||||||||||||||||||||||||||||||||||
function onMountVehicle(%datablock, %obj, %vehicle)
{
   // Is the vehicle currently mountable?
   if (%vehicle.mountable == false)
   {
      return;
   }
   if (%vehicle.getTeam() != 0)
     {
      if (%vehicle.getTeam() != %obj.client.team.teamID)
        {
         messageClient(%obj.client, 'MsgVehicle', '\c0Cannot mount other team\'s vehicle');
         return;
        }
     }


   %vel = %vehicle.getVelocity();
   %speed = vectorLen(%vel);

   // Find an empty seat
   %seat = findEmptySeat(%vehicle, %datablock);
   %obj.isMounted = true;
   %client = %obj.client;
   setActiveSeat(%client, %vehicle, %datablock, %seat);
   %vehicle.setTeam(%client.team.teamID);
   %vehicle.client = %client;


}
// *******************************************
// Function: setActiveSeat
//
// Inputs: Client connection object
//         Vehicle object
//         Vehicle datablock
//         Seat number
//
// Outputs: none
//
// This function re-seats the player into the
// given seat. The player is rotated to face
// the direction defined for the seat. The
// player's pose is also modified and the
// client is ordered to use the correct
// movement map.
// *******************************************
//||||||||||||||||||||||||||||||||||||||||||||||||||||
//   called from  serverCmdFindNextFreeSeat in commands.cs
//||||||||||||||||||||||||||||||||||||||||||||||||||||
function setActiveSeat(%client, %vehicle, %vehicleblock, %seat)
{
   %obj = %client.player;
   %vehicle.mountObject(%obj,%vehicle.seat[%seat]);
   %obj.setTransform(%vehicleblock.mountPointTransform[%seat]);
   %obj.setActionThread(%vehicle.getDatablock().mountPose[%seat],true,true);

   %client.player.mSeat = %seat;
   %client.player.mVehicle = %vehicle;

}

// *******************************************
// Function: onPlayerVehicle
//
// Inputs: Player object
//         Player object
//         Vehicle object
//         Seat (node)
//
// Outputs: None
//
// This function is called when the player
// mounts a vehicle. It commands the client to
// display the correct movement map, removes
// the player's weapon and gives them control
// of the vehicle if they are in the driver's
// seat.
// *******************************************

// *******************************************
//||||||||||||||||||||||||||||||||||||||||||||||||||||
//   called from  Armor::onMount in player.cs
//||||||||||||||||||||||||||||||||||||||||||||||||||||
function onPlayerMount(%player,%obj,%vehicle,%seat)
{
  %obj.lastWeapon = %obj.getMountedImage($WeaponSlot);
  %obj.unmountImage($WeaponSlot);
  %obj.setActionThread(%vehicle.getDatablock().mountPose[%seat],true,true);
  %obj.setArmThread("head");
  %vehicle.onEnterVehicle(%obj.client, %seat ); // Handles GUI additions, et al
  %vehicle.initialAmmo(%obj.client, %seat);
  CommandToClient(%obj.client, 'SetCustomCrossHair', "none");

  // set client for turfet if any
  if ( %vehicle.turretNode[%seat])
    %vehicle.turretNode[%seat].client = %obj.client;
}

//dummy callbacks
function ShapeBase::onEnterVehicle(%client, %node){
//here to stop console spam
}

function ShapeBase::initialAmmo(%client){
//here to stop console spam
}

// *******************************************
// Function: doPlayerDismount
//
// Inputs: Player object
//         Player datablock
//         Forced flag
//
// Outputs: None
//
// This function is called when the player
// presses the dismount key. The player can only
// dismount when the vehicle is stationery.
// *******************************************
function doPlayerDismount(%player, %obj, %forced)
{
   %vel = %obj.getVelocity();
   %speed = vectorLen(%vel);

   %obj.mVehicle.onExitVehicle(%obj.client); // Handles GUI removal, et al

   if (!%obj.isMounted())
   {
      // The vehicle is moving or the player is not mounted.
      return;
   }

   // shut off weapon if any
   if (isObject(%obj.mVehicle.turretNode[%obj.mSeat]) )
     {
      %obj.mVehicle.turretNode[%obj.mSeat].setImageTrigger(0,0);
      %obj.mVehicle.turretNode[%obj.mSeat].setImageTrigger(1,0);
     }

   // Find the position above dismount point.
   %pos    = getWords(%obj.getTransform(), 0, 2);
   %oldPos = %pos;
   %vec[0] = " 1.2  0  0";
   %vec[1] = " 0  -1.2  0";
   %vec[2] = " 0  1.2  0";
   %vec[3] = "-1.2  0  0";
   %vec[4] = " 4  0  0";
   %impulseVec  = "0 0 0";
   %vec[0] = MatrixMulVector( %obj.getTransform(), %vec[0]);

   // Make sure the point is valid
   %pos = "0 0 0";
   %numAttempts = 5;
   %success     = -1;
   for (%i = 0; %i < %numAttempts; %i++) {
      %pos = VectorAdd(%oldPos, VectorScale(%vec[%i], 7));
      if (%obj.checkDismountPoint(%oldPos, %pos)) {
         %success = %i;
         %impulseVec = %vec[%i];
         break;
      }
   }

   if (%forced && %success == -1)
      %pos = %oldPos;

   // Schedule the function to set the mount flag, so that the player
   // can mount another vehicle in the future.
   %obj.schedule(1000, "MountVehicles", true);

   %obj.unmount();
   %obj.setControlObject(%obj);
   %obj.mountVehicle = false;

   // Check to see if anyone is still in this vehicle, and if not - reset the teamID
   if (!isVehicleOccupied(%obj.mVehicle))
     {
      %obj.mVehicle.setTeam(0);
     }

   %obj.mVehicle = 0;

   // Position above dismount point
   %obj.setTransform(%pos);
   %obj.applyImpulse(%pos, VectorScale(%impulseVec, %obj.getDataBlock().mass));

   %obj.setActionThread("run",true,true);

   %weaponimage = %obj.getMountedImage(0);
   if (%weaponimage.customLookAnim !$= "")
   {
      %obj.setArmThread(%weaponimage.customLookAnim);
   }
   else
   {
      %obj.setArmThread("looknife");
   }
}

// return true if %vehicle has a player mounted in it
function isVehicleOccupied(%vehicle)
  {
   for (%slot =0; %slot <= 7; %slot++)
      {
       %onBoard = %Vehicle.getMountedObject(%slot);
       if (isObject(%onBoard) )
         if ( %onBoard.getClassName() $= "Player")
           {
            return true;
           }
      }

   return false;
  }


// *******************************************
// Function: onPlayerUnmount
//
// Inputs: Player object
//         Player datablock
//         Vehicle object
//         Seat
//
// Outputs: None
//
// This function mounts the player's weapon
// after they dismount from the vehicle.
// *******************************************
function onPlayerUnmount(%player, %obj, %vehicle, %node)
{
   if ( %vehicle.turretNode[%seat])
     %vehicle.turretNode[%seat].client = 0;

   %obj.mountImage(%obj.lastWeapon, $WeaponSlot);
}

// *******************************************
// Function: findNextFreeSeat
//
// Inputs: Client connection object
//         Vehicle object
//         Vehicle datablock
//
// Outputs: FALSE if a seat is not found
//          The next free seat number
//
// This function searches the vehicle for a
// free seat.
// *******************************************
function findNextFreeSeat(%client, %vehicle, %vehicleblock)
{
      %hasparent = %vehicle.getobjectmount();
      if(%hasparent != 0 )
      {
         %found = true;
         %seat = "Vehicle";
         return %seat;
      }

   // Check the next seat
   %seat = %client.player.mSeat + 1;
   if (%seat == %vehicleblock.numMountPoints)
   {
      // Check from seat 0, we've run out of seats to check.
      %seat = 0;
   }

   if(!isObject(%vehicle)){
      //dont know how we got here, but we dont have a vehicle
      return;
   }

   // Reset the flag.
   %found = false;

   %mountSearch = 0;
   // Search through the seats.
   while ((%seat != %client.player.mSeat) && (%found == false))
   {
      %mountSearch++;

      //we have looked too many times, lets get outta here and not inf loop
      if(%mountSearch>%vehicleblock.numMountPoints)
         return -1;

      //this covers us if we were in a higer number seat and wrap around
      if (%seat >= %vehicleblock.numMountPoints)
      {
         // Go back to the first (driver's) seat
         %seat = 0;
      }

      // Is this seat free?
      %node = %vehicle.getMountNodeObject(%vehicle.seat[%seat]);
      if (%node == 0)
        {
         // Yes it is free.
         return %seat;
        }

      if (%found == false)
      {
         // We couldn't find a free seat.
         %seat++;
      }
   }

   if (%found == false)
      return -1;
   else
      return %seat;
}

// *******************************************
// Function: findPreviousFreeSeat
//
// Inputs: Client connection object
//         Vehicle object
//         Vehicle datablock
//
// Outputs: FALSE if a seat is not found
//          The previous free seat number
//
// This function searches the vehicle for a
// free seat. It searches in the opposite
// direction to the findNextFreeSeat function.
// *******************************************
function findPreviousFreeSeat(%client, %vehicle, %vehicleblock)
{
   // Check the previous seat
   %seat = %client.player.mSeat - 1;

   if (%seat == -1)
   {
      // Check from the last seat
   }

   %found = false;

   // Search through the seats.
   while ((%seat != %client.player.mSeat) && (%found == false))
   {
      if (%seat < 0)
      {
         %seat = %vehicleblock.numMountPoints - 1;
      }

      %node = %vehicle.getMountNodeObject(%vehicle.seat[%seat]);

      if (%node == 0)
      {
         return %seat;
      }
      if (%found == false)
         %seat--;
   }

   if (%found == false)
      return -1;
   else
      return %seat;
}



// *******************************************
// Function: isVehicleMoving
//
// Inputs: Vehicle object
//
// Outputs: FALSE if the vehicle is NOT moving
//          TRUE if the vehicle is moving
//
// This function calcuates if the vehicle is
// moving according to a threshold value
// defined on a per-vehicle basis.
// *******************************************
function isVehicleMoving(%vehicle)
{
   // Calculate the vehicle's velocity
   %vel = %vehicle.getVelocity();
   %speed = vectorLen(%vel);

   // Determine if the vehicle is moving according to the
   // threshold value defined in the vehicle's datablock.
   if (%speed > %vehicle.getDataBlock().stationaryThreshold)
      return true;
   else
      return false;
}

// *******************************************
// Function: getVehicleSpeed
//
// Inputs: Vehicle object
//
// Outputs: Vehicle's speed
//
// This utility function calculates the
// vehicle's velocity. Note that the speed
// does not define the vehicle's direction.
// *******************************************
function getVehicleSpeed(%vehicle)
{
   %vel = %vehicle.getVelocity();
   %speed = vectorLen(%vel);
   return %speed;
}

// *******************************************
// Function: dumpMounts
//
// Inputs: Vehicle object
//         Vehicle datablock
//
// Outputs: None
//
// This utility function simply dumps the
// mounts for a given vehicle. It is used to
// check which players the game engine thinks
// are currently mounted for each seat.
// *******************************************
function dumpMounts(%vehicle, %vehicleBlock)
{
   echo("**************");
   echo("Dumping mounts");
   echo("--------------");
   for (%ii=0; %ii<%vehicleblock.numMountPoints;%ii++)
   {
     echo(%ii @ ": " @ %vehicle.getMountNodeObject(%ii));
   }
   echo("**************");
}

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
function VehicleData::onRemove(%this, %obj)
{
   // free mounted turret
   if (isObject(%obj.mountedTurret))
   {
      %obj.mountedTurret.schedule(100,"delete"); // Rex- default= 300 I changed this to get the turrets deleting with vehicle
      MissionCleanup.Add(%obj.mountedTurret);    // I think debris is landing on the bounding box
   }
}

// damage the vehicle
// do armor calculations and any damage reduction here
// a simple armor system has been put in, but there is no provision for Armor piercing damage
// Damage type is passed in, but not used -- this is where to put in checks for various
// types of damages and resistances.
function Vehicle::damage(%this, %obj, %pos, %damage, %damageType)
  {
   // %this = vehicle getting damaged
   // %obj  = source of the damage (not always filled in)
   // %pos  = position of the damage giver
   // %damage = amount of dmage to do
   // %damageType = type of damage being done
   if ( EditorGui.active == true) return;

   %db = %this.getDataBlock();

   if (%this.getDamageLevel() >= %db.destroyedLevel)
     {
      return;
     }
   echo(%db.getName() @ " takes " @ %damage @ " damage from " @ %obj.client.nameBase @ " type: "@ %damageType @ " dmgLevel: " @ %this.getDamageLevel() );

   // check to see if damage bounces off
   if (%db.armorvalue >= %damage) return;

   // subtract armor value from incomming damage & apply the result
   %value = %damage - %db.armorValue;
   %clients = %this.getMountedClients();
   %this.applyDamage(%value);
   if (%this.getDamageLevel() >= %db.destroyedLevel)
     {
      if (%obj.client)
        messageAll('MsgTeamActivity', %obj.client.nameBase @ " destroyed a " @ %db.EnglishName);

      playWarn(%this, %obj );
      schedule($Vehicle::WarningTimer, 0, doVehicleDestroyed, %db, %this, %obj.client);
     }
  }

function doVehicleDestroyed(%vehDB, %vehicle, %client)
  {
   // vehicle is destroyed.. everyone on board gets the same
   for ( %slot=0; %slot <=7; %slot++)
     {
      %onBoard = %vehicle.getMountedObject(%slot);
      if (isObject(%onBoard.client) )
        {
         if (%onBoard.getClassName() $= "Player")
           {
            // schedule a dismount for each object because the order of the mounted slots
            // will change after doing a dismount causing all sorts of wierd bugs
            schedule(0,0,"dismountAndKill", %vehDB, %vehicle, %onBoard, %client);
           }
        }
     }

   radiusDamage(%vehicle,%vehicle.getPosition() ,%vehDB.damageRadius,%vehDB.radiusDamage,"Explosion",%vehDB.areaImpulse);
   %vehicle.setDamageState(Destroyed);
  }

function dismountAndKill(%vehDB, %vehicle, %player, %client)
  {
   doPlayerDismount(%vehicle, %player, 1);
   // use "override" damage type here so bad drivers will be compensated
   %player.damage(%client, "0 0 0", %player.getDataBlock().maxDamage * 2, "Override" );
  }

function WheeledVehicleData::onTrigger(%data, %obj, %trigger, %state)
{
   // data = datablock
   // obj = object number
   // trigger = 0 for "fire", 1 for "jump", 3 for "thrust"
   // state = 1 for firing, 0 for not firing
   if(%trigger == 0)
   {
      switch (%state) {
         case 0:
            %obj.fireWeapon = false;
            %obj.setImageTrigger(0, false);//false
            %obj.setImageTrigger(2, false);//false
         case 1:
            %obj.fireWeapon = true;
            if (isObject(%obj.mountedTurret))
              {
               // "pass" in the client controlling this turret so damage credit can be handed out
               %obj.mountedTurret.client = %obj.getMountedObject(0).client;
               %obj.mountedTurret.fire(0);
            } else {
               if(%obj.nextWeaponFire == 2) {
                  %obj.setImageTrigger(0, true);
                  %obj.setImageTrigger(2, false);//false
               }
               else {
                  %obj.setImageTrigger(0, false);
                  %obj.setImageTrigger(2, true);//true
               }
            }
      }
   }
}

// support function to prevent console error spam
function Vehicle::getState()
  {
   return 0;
  }
function playWarn(%this, %obj)
{
   %this.playAudio(0,WarningSound);
   %this.schedule($Vehicle::WarningTimer, "stopAudio", 0);
}

// Handle a vehicle running into something -- this is different from a collision in
// that the vehicle is travelling some speed. see "minImpactSpeed" in the vehicle datablocks
// to change the min speed at which this gets called
function doVehicleImpact(%this, %obj, %collide, %vec, %vecLen)
  {
   // %this = datablock of vehicle
   // %obj = instance of the vehicle
   // %collide = not sure- this is seldom given a value
   // %vec = direction of the impact
   // %vecLen = impact speed
   if ( EditorGui.active == true) return;

   // hack so vehicles wont take impact damage from hitting players
   // this is needed because %collide is never filled in with what we ran into.
   if (%this.collidePlayer == 1)
     {
      %this.collidePlayer = 0;
      return;
     }

   %db = %obj.getDataBlock();
   if (%obj.getDamagePercent() == 1) return;

   // maxImpactSpeed  is an added value in the vehicle datablock
   // when a vehicle impacts something at greater than maxImpactSpeed, its destroyed.
   if ( %vecLen >= %db.maxImpactSpeed)
     {
      %obj.damage(%obj, %vec, %db.maxDamage * 2, "Impact");
      return;
     }
   %obj.damage(%obj, %vec, %vecLen * %obj.getDataBlock().speedDamageScale, "Impact" );
  }

// Vehicle has run into something
// if it was moving fast enough, then damage is done to whatever was run into
// damage is based on speed of the collision, and mass of the vehicle.
function doVehicleCollision(%this, %obj, %col, %vec, %Speed)
  {
   // %this = datablock of the moving vehicle
   // %obj = instance of the moving vehicle
   // %col = thing that was run into
   // %vec = velocity vector of the collision

   // if either of the things colliding are dead, then dont do anything
   if (%obj.getDamagePercent() == 1) return;
   if (%col.getDamagePercent() == 1) return;

   %Testspeed = VectorLen(%obj.getVelocity());   // if %obj is effectively sitting still, do nothing
   if (%Testspeed < 0.8) return;


   // check min collision speed of the thing we ran into
   if (%speed > %col.getDataBlock().minCollisionSpeed)
     {
      if (%col.getClassName() $= "Player") %this.collidePlayer = 1;
      %col.damage(%obj, %this , %speed * %this.collisionMass * 0.25, "Collision");

      // if whatever we hit is still alive, then apply possible knockback damage.
      if ( %col.getDamagePercent() < 1 )
        {
         %db = %col.getDataBlock();
         if ( %this.collisionMass < (%db.collisionMass * 0.75) )
           {
            // vehicle ran into something bigger than it was,  some damage will come back
            %obj.damage(%col, %this, %db.collisionMass * %speed * 0.125, "Knockback");
           }
        }
     }

  }

// called to respawn a vehicle at its old location when its destroyed.
function RespawnVehicle(%className, %dataBlock, %pos, %rot)
  {
   // create a new vehicle with the given datablock
   %code = "return new " @ %className @ "() {";
   %code = %code @ " position = \"" @ %pos @ "\"; ";
   %code = %code @ " rotation = \"" @ %rot @ "\"; ";
   %code = %code @ " dataBlock = \"" @ %dataBlock @ "\"; ";
   %code = %code @ " scale = \"1 1 1\"; ";
   %code = %code @ "};";
   eval(%code);
  }

// Vehicle specifics upon entering. We always pop other mappings, as this function will be called upon changing
// seats, as well as upon initial entering.

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// called from onPlayerMount via Armor::onMount in player.cs
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
function Vehicle::onEnterVehicle(%this, %client, %seat)
{
   %datablockName = %client.player.mVehicle.getDataBlock().getName();
  // we can use the %datablockName variable to push different action maps
  // for each Vehicle type)

  if (%seat == 0)
  {
     commandToClient(%client, 'CreateGenericVehicleGui', 1, %seat, %this);
     commandToClient(%client, 'PushGenericDriverMap');
     commandToClient(%client, 'PopGenericPassengerMap');
  }
  else
  {
     if ( %this.turretNode[%seat])
       commandToClient(%client, 'CreateGenericVehicleGui', 1, %seat, %this, 1);
     else
       commandToClient(%client, 'CreateGenericVehicleGui', 1, %seat, %this, 0);

     commandToClient(%client, 'PushGenericPassengerMap');
     commandToClient(%client, 'PopGenericDriverMap');
  }
  commandToClient(%client, 'PopActionMap');
}

function Vehicle::onExitVehicle(%this, %client)
{
   %datablockName = %client.player.mVehicle.getDataBlock().getName();
  // we can use the %datablockName variable to push/pop different action maps
  // for each Vehicle type)

  commandToClient(%client, 'CreateGenericVehicleGui', 0);
  commandToClient(%client, 'PopGenericDriverMap');
  commandToClient(%client, 'PopGenericPassengerMap');
  commandToClient(%client, 'PushActionMap');
}

function ServerCmdPowerUp(%client)
{
	 // Come back to this when I run across how to slow->stop an animation( and VV), rather than doing so abruptly
	 // on second thought, wtf are we doing it here, rather than in code with all the other anims? Or should they all be
	 // scripted as well?!?
	 %vehicle = %client.player.mvehicle;
	 if (%vehicle) {
	 	 %state = %vehicle.getEngineState();
	 	 if (!%state)
	 	 {
	     %vehicle.setEngineState(true);   ///< Engine state, can't do much with it off
	     %vehicle.setGravity(-10);
	   }
	   else
	   {
	     %vehicle.setEngineState(false);
	     %vehicle.setGravity(-20);
	   }
	 }
}

function VehicleData::onDestroyed(%data, %obj, %prevState)
{

}

// Rob - If a vehicle is targetted by a rocket launcher, this fires
// It checks to see if there is a pilot, and if so sends a command
// to pop up a warning.
function Vehicle::OnTargetted(%vehicle, %targettingPlayer)
{
   echo("Targeted " @ %vehicle SPC %vehicle.getDatablock().getName() );

   //Get our vehicles pilot ID
   %pilot = %vehicle.getMountNodeObject(%vehicle.seat[0]);
   if (%pilot == %targettingPlayer)
     return;

   SetTargetLock(%vehicle, %targettingPlayer);

   //If there IS an enemy pilot, warn him quick!
   if (%pilot)
     {
      if (($Game::FriendlyFire == 1) || (%pilot.client.TeamID != %targettingPlayer.client.TeamID) )
        {
         commandToClient(%pilot.client, 'vehicletargetted');
        }
     }
}

function VehicleData::onUnderWater(%data, %obj)
{
  %obj.damage(%obj, %obj.getTransform(),100000, "Water");
}

// return a TDL of client IDs that are mounted on this vehicle
// used to apply kill and damage credits
function Vehicle::getMountedClients(%this)
  {
   %found = false;
   for (%slot =0; %slot <= 7; %slot++)
     {
      %onBoard = %this.getMountedObject(%slot);
      if (isObject(%onBoard) )
        {
         if (%onBoard.client > 0)
           {
            if (!%found)
              {
               %string = %onBoard.client.getID();
               %found = true;
              }
            else
               %string = %string TAB %onBoard.client.getID();
           }
        }
     }
   if (!%found)
      return 0;

   return %string;
  }
