//-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// ============================================================
// Copyright 2007 Dream Games, Inc.
// Combat Starter Kit
// ============================================================

//-----------------------------------------------------------------------------

// Information extacted from the shape.
//
// Wheel Sequences
//    spring#        Wheel spring motion: time 0 = wheel fully extended,
//                   the hub must be displaced, but not directly animated
//                   as it will be rotated in code.
// Other Sequences
//    steering       Wheel steering: time 0 = full right, 0.5 = center
//    breakLight     Break light, time 0 = off, 1 = breaking
//
// Wheel Nodes
//    hub#           Wheel hub, the hub must be in it's upper position
//                   from which the springs are mounted.
//
// The steering and animation sequences are optional.
// The center of the shape acts as the center of mass for the car.

//-----------------------------------------------------------------------------


datablock AudioProfile(JeepEngineSound)
{
   filename    = "~/data/sound/combat/engine_idle.ogg";
   description = AudioClosestLooping3d;
   preload     = true;
};

datablock AudioProfile(SoftImpactSound)
{
   filename    = "~/data/sound/combat/m_crunch.ogg";
   description = AudioClose3d;
   preload     = true;
};

datablock AudioProfile(HardImpactSound)
{
   filename    = "~/data/sound/combat/crash.ogg";
   description = AudioClose3d;
   preload     = true;
};

datablock AudioProfile(SquealSound)
{
   filename    = "~/data/sound/combat/screech.ogg";
   description = AudioClose3d;
   preload     = true;
};

datablock ParticleData(TireParticle)
{
   textureName          = "~/data/shapes/combat/dustParticle";
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 400;
   colors[0]     = "0.46 0.36 0.26 1.0";
   colors[1]     = "0.46 0.46 0.36 0.0";
   sizes[0]      = 1.0;
   sizes[1]      = 4.0;
};

datablock ParticleEmitterData(TireEmitter)
{
   ejectionPeriodMS = 20;
   periodVarianceMS = 10;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "TireParticle";
};

datablock DebrisData(BaseWheeledVehicleDebri)
{
	explodeOnMaxBounce = false;

	elasticity = 0.15;
	friction = 0.5;

	lifetime = 10.0;
	lifetimeVariance = 5.0;

	minSpinSpeed = 5;
	maxSpinSpeed = 600;

	numBounces = 5;
	bounceVariance = 0;

        staticOnMaxBounce = true;
        snapOnMaxBounce = true;
        gravModifier = 2.0;

	useRadiusMass = true;
	baseRadius = 1;

        velocity = 30.0;
	velocityVariance = 12.0;
};


//----------------------------------------------------------------------------

datablock WheeledVehicleTire(BaseWheeledVehicleTire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = "~/data/shapes/combat/wheel.dts";
   staticFriction = 0.8;
   kineticFriction = 0.9;

   // Spring that generates lateral tire forces
   lateralForce = 18000;
   lateralDamping = 6000;
   lateralRelaxation = 1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 18000;
   longitudinalDamping = 5000;
   longitudinalRelaxation = 1;
};

datablock WheeledVehicleSpring(BaseWheeledVehicleSpring)
{
   // Wheel suspension properties
   length = 0.65;             // Suspension travel
   force = 1800;              // Spring force
   damping = 1000;            // Spring damping
   antiSwayForce = 6;         // Lateral anti-sway force

};

datablock WheeledVehicleData(BaseWheeledVehicle)
{
   category = "Vehicles";
   shapeFile = "~/data/shapes/combat/humvee.dts";
   emap = true;
   className = BaseWheeledVehicleData;

   mountable     = true;
   maxMountSpeed = 100.0;
   mountDelay    = 0;
   dismountDelay = 10.5;

   maxDamage = 925.0;
   destroyedLevel = 925.0;

   renderWhenDestroyed = true;
   maxSteeringAngle = 0.985;  // Maximum steering angle, should match animation
   tireEmitter = TireEmitter; // All the tires use the same dust emitter

   // 3rd person camera settings
   cameraRoll = false;         // Roll the camera with the vehicle
   cameraMaxDist = 10.8;         // Far distance from vehicle
   cameraOffset = 3.5;        // Vertical offset from camera mount point
   cameraLag = 0.26;           // Velocity lag of camera
   cameraDecay = 1.25;        // Decay per sec. rate of velocity lag

   // Rigid Body
   mass = 180;
   massCenter = "0 -1.8 0";    // Center of mass for rigid body
   massBox = "0 0 0";         // Size of box used for moment of inertia,
                              // if zero it defaults to object bounding box
   drag = 0.8;                // Drag coefficient
   bodyFriction = 0.7;
   bodyRestitution = 0.6;
   minImpactSpeed = 5;        // Impacts over this invoke the script callback
   softImpactSpeed = 5;       // Play SoftImpact Sound
   hardImpactSpeed = 15;      // Play HardImpact Sound

   // Integration is a count of how many times TGE will run
   // the vehicle physics in each tick.  Set it as low as possible
   // because physics is cpu intensive
   integration = 4;           // Physics integration: TickSec/Rate
   collisionTol = 0.1;        // Collision distance tolerance
   contactTol = 0.01;          // Contact velocity tolerance

   // Engine
   engineTorque = 185;       // Engine power
   maxRevs = 71.0;           // percentage value indicating how high the engine can rev
   // a space separated list of float values to multiply the engineTorque with.  Max of 12 gears allowed
   Gears = "3.3 3.0 2.1 1.35 1.0 0.8";
   engineBrake = 200;        // Braking when throttle is 0
   brakeTorque = 1000;        // When brakes are applied
   maxWheelSpeed = 30;        // Engine scale by current speed / max speed

   maxEnergy = 50;               //Afterburner and any energy weapon pool. Gasoline.
   rechargeRate = 0;              //Solar batteries recharging our engines automatically?

   jetForce = 3000;
   minJetEnergy = 30;
   jetEnergyDrain = 2;

   // Sounds
     engineSound = JeepEngineSound;
   squealSound = SquealSound;
   softImpactSound = SoftImpactSound;
   hardImpactSound = HardImpactSound;

   // Explosion
   directDamage = 450;
   damageRadius = 25;
   radiusDamage = 50;
   areaImpulse  = 500;

   explosion       = VehicleExplosion;

   debris          = BaseWheeledVehicleDebri;
   damageEmitter[0]        = LightDamageEmitter;
   damageEmitter[1]        = HeavyDamageEmitter;
   damageEmitter[2]        = DestroyedEmitter;
   damageEmitterOffset[0]  = "0.0 -3.0 0.0 ";
   damageLevelTolerance[0] = 0.3;
   damageLevelTolerance[1] = 0.7;
   numDmgEmitterAreas      = 1;

   // Starter.Combat values
   maxImpactSpeed = 40;       // if we hit something above this speed, the car is destroyed
   minCollisionSpeed = 6;     // anything colliding with this vehicle above this speed does damage, otherwise no effect
   speedDamageScale = 3;      // a measure of how much damage to take when we crash into something
   armorValue = 20;           // damage must be above this amount to have any effect.
   steeringDecay = 0.95;      // Scale the steering by this amount every tick, effective moves the steering to zero
   EnglishName = "WheeledVehicle";
   RespawnTimer = 60;         // seconds after being destroyed

   // Damage location details
   boxTopPercentage       = 0.50;
   boxBottomPercentage    = 0.50;
   boxTopLeftPercentage   = 0;
   boxTopRightPercentage  = 1;
   boxTopBackPercentage   = 0;
   boxTopFrontPercentage  = 1;
};


//-----------------------------------------------------------------------------
function WheeledVehicleData::create(%block)
  {
   if ( %block $= "Tank01")
     {
      return Tank01::create(%block);
     }
   %obj = new WheeledVehicle() {
      dataBlock = %block;
   };
   return(%obj);
}


//-----------------------------------------------------------------------------
function WheeledVehicleData::onAdd(%this,%obj)
{
   %obj.mountable = true;
   %obj.setEnergyLevel(%this.MaxEnergy);
   %obj.setRechargeRate(%this.rechargeRate);
   %obj.setGravity(-20);
   %obj.weaponviews = 0; //used in player.cs onMount
   %obj.SpawnPos = %obj.position;
   %obj.SpawnRot = %obj.rotation;
}

//-----------------------------------------------------------------------------
function WheeledVehicleData::onRemove(%this, %obj)
{
   // free mounted turret
   if (isObject(%obj.mountedTurret))
   {
	%obj.mountedTurret.delete();
   }
}


//-----------------------------------------------------------------------------
// send the initial ammo settings to %client
function WheeledVehicle::initialAmmo(%this, %client, %seat)
{
   %datablock = %client.player.mVehicle.getDataBlock();
   %datablockName = %datablock.getName();

  if (%datablockName $= "Jeep")
    Jeep::initialAmmo(%this, %client, %seat);

  if (%datablockName $= "Tank01")
    Tank01::initialAmmo(%this, %client, %seat);

  if (%datablockName $= "Stryker")
    Stryker::initialAmmo(%this, %client, %seat);

}


